"""
airtest核心api的二次封装
"""
from airtestProject.airtest.core import api as air

# from airtestProject.commons.utils.MyCv import MyTemplate
from airtestProject.commons.utils.logger import log
import time


def init_device(platform=None, uuid=None, **kwargs):
    try:
        air.init_device(platform=platform, uuid=uuid, **kwargs)
        log.info(f"init device success, platform: {platform}, uuid:  {uuid}")
    except Exception as e:
        log.error(f"failed to init device, platform: {platform}, uuid:  {uuid}")
        raise e


def connect_device(uri):
    log.info('🍇 🍉 ready to connect device host ...')
    try:
        air.connect_device(uri)
        log.info(f'🍊 🍋 connect device succeeded... host: {uri}')
    except ConnectionError as e:
        log.error(f'connect device failed... host: {uri}')
        raise ConnectionError from e


def device():
    current_active_device = air.device()
    log.info(f'get current active device: {current_active_device}')
    return current_active_device


def set_current(idx):
    try:
        log.info(f"set current active device idx: {idx}")
        air.set_current(idx)
    except Exception as e:
        log.error(f"failed to set current active device idx: {idx}")
        raise e


def auto_setup(basedir=None, devices=None, logdir=None, project_root=None, compress=None):
    air.auto_setup(basedir=basedir, devices=devices, logdir=logdir, project_root=project_root, compress=compress)


def shell(cmd):
    try:
        log.info(f"execute adb shell command: {cmd}")
        return air.shell(cmd)
    except Exception as e:
        log.error(f"failed to execute adb shell command: {cmd}")
        raise e


def start_app(package, activity=None):
    try:
        air.start_app(package, activity)
        log.info(f"start app: {package}, activity: {activity}")
    except Exception as e:
        log.error(f"failed start app {package}, activity: {activity}")
        raise e


def stop_app(package):
    try:
        air.stop_app(package)
        log.info(f"stop the application on device, package: {package}")
    except Exception as e:
        log.error(f"failed to stop the application on device, package: {package}")
        raise e


def clear_app(package):
    try:
        air.clear_app(package)
        log.info(f"clear data of the application on device, package: {package}")
    except Exception as e:
        log.error(f"failed to clear data of the application on device, package: {package}")
        raise e


def install(filepath, **kwargs):
    try:
        log.info(f"install application: {filepath}")
        return air.install(filepath, **kwargs)
    except Exception as e:
        log.error(f"failed to install application: {filepath}")
        raise e


def uninstall(package):
    try:
        log.info(f"uninstall application: {package}")
        return air.uninstall(package)
    except Exception as e:
        log.error(f"failed to uninstall application: {package}")
        raise e


def snapshot(filename=None, msg="", quality=None, max_size=None):
    try:
        log.info(f"take the screenshot of the device and save it to the filepath:{filename}.")
        return air.snapshot(filename, msg, quality, max_size)
    except Exception as e:
        log.error(f"failed to wake up and unlock the device")
        raise e


def wake():
    try:
        log.info(f"wake up and unlock the device")
        return air.wake
    except Exception as e:
        log.error(f"failed to wake up and unlock the device")
        raise e


def home(package):
    raise NotImplementedError


def touch(v, times=1, **kwargs):
    try:
        log.info(f"perform the touch action on the device screen, pos: {v}")
        v = MyTemplate(v)
        return air.touch(v, times, **kwargs)
    except Exception as e:
        log.error(f"failed to perform the touch action on the device screen, pos: {v}")
        raise e


def double_click(v1, v2=None, vector=None, **kwargs):
    log.info(f"perform double click")
    return air.double_click(v1, v2, vector, **kwargs)


def swipe(v1, v2=None, vector=None, **kwargs):
    log.info("Perform the swipe action on the device screen.")
    return air.swipe(v1, v2, vector, **kwargs)


def pinch(in_or_out='in', center=None, percent=0.5):
    log.info(f"perform the pinch action on the device screen")
    return air.pinch(in_or_out, center, percent)


def keyevent(keyname, **kwargs):
    log.info(f"perform key event keyname: {keyname} on the device.")
    return air.keyevent(keyname, **kwargs)


def text(value, enter=True, **kwargs):
    try:
        log.info(f"input {value} on the target device")
        return air.text(text, enter, **kwargs)
    except Exception as e:
        log.error(f"failed to input {value} on the target device")
        raise e


def sleep(secs=1.0):
    time.sleep(secs)
    log.info(f'time sleep {secs} seconds')


def wait():
    raise NotImplementedError


def exists(v):
    try:
        log.info(f"the node: {v} exists.")
        return air.exists(v)
    except Exception as e:
        log.error(f"the node: {v} not exists.")
        raise e


if __name__ == '__main__':
    sleep()


# _*_ coding:UTF-8 _*_
import time
import functools


def retries(max_tries, delay=1, backoff=2, exceptions=(Exception,), hook=None):
    """
    Function decorator implementing logic to recover from fatal errors. If a function fails to call due to any
    fatal error, the decoration tries to call it again after given delay time.

    The call delay time is counted as follows:
    delay * backoff * number of attempts to call the function after its failure

    It is possible to specify the custom tuple of exception classes within the 'exceptions' parameter. Only if such
    exception is detected, the function retries to call itself again.

    It is also possible to specify a hook function (with number of remaining re-tries and exception instance) which
    will be called prior the retrying attempt. Using the hook function gives the possibility to log the failure.
    Hook function is not called after failures or when no attempts left.

    Args:
        max_tries: maximum number of attempts to call the function, the decorator will call the function up
                   to max_tries, if all attempts fails, then the exception is risen
        delay: parameter to count the sleep time
        backoff: parameter to count the sleep time
        exceptions: A tuple of exception classes; default (Exception,)
        hook: A function with the signature myhook(tries_remaining, exception);
              default value is None

    Raises:
        Exception class and subclasses by default

    Returns:
        wrapper

    """

    def dec(func):
        @functools.wraps(func)
        def f2(*args, **kwargs):
            mydelay = delay
            tries = range(max_tries)
            # support Python conver range obj to list obj
            tries = list(tries)

            tries.reverse()
            for tries_remaining in tries:
                try:
                    return func(*args, **kwargs)
                except exceptions as e:
                    if tries_remaining > 0:
                        if hook is not None:
                            hook(tries_remaining, e, mydelay)
                        time.sleep(mydelay)
                        mydelay = mydelay * backoff
                    else:
                        raise
                else:
                    break

        return f2

    return dec



# coding=utf-8

from airtestProject.poco.pocofw import Poco
from airtestProject.poco.agent import PocoAgent
from airtestProject.poco.drivers.std.attributor import StdAttributor
from airtestProject.poco.drivers.std.dumper import StdDumper
from airtestProject.poco.drivers.std.screen import StdScreen
from airtestProject.poco.drivers.std.inputs import StdInput
from airtestProject.poco.freezeui.hierarchy import FrozenUIHierarchy
from airtestProject.poco.utils.airtest import AirtestInput
from airtestProject.poco.utils.simplerpc.rpcclient import RpcClient
from airtestProject.poco.utils.simplerpc.transport.tcp.main import TcpClient
from airtestProject.poco.utils.simplerpc.utils import sync_wrapper
from airtestProject.poco.utils.device import default_device

from airtestProject.airtest.core.api import connect_device, device as current_device
from airtestProject.airtest.core.helper import device_platform
import socket


__all__ = ['StdPoco', 'StdPocoAgent']
DEFAULT_PORT = 15004
DEFAULT_ADDR = ('localhost', DEFAULT_PORT)


class StdPocoAgent(PocoAgent):
    def __init__(self, addr=DEFAULT_ADDR, use_airtest_input=True):
        self.conn = TcpClient(addr)
        self.c = RpcClient(self.conn)
        self.c.DEBUG = False
        self.c.connect()

        hierarchy = FrozenUIHierarchy(StdDumper(self.c), StdAttributor(self.c))
        screen = StdScreen(self.c)
        if use_airtest_input:
            inputs = AirtestInput()
        else:
            inputs = StdInput(self.c)
        super(StdPocoAgent, self).__init__(hierarchy, inputs, screen, None)

    @property
    def rpc(self):
        return self.c

    @sync_wrapper
    def get_debug_profiling_data(self):
        return self.c.call("GetDebugProfilingData")

    @sync_wrapper
    def get_sdk_version(self):
        return self.c.call('GetSDKVersion')


class StdPoco(Poco):
    """
    Poco standard implementation for PocoSDK protocol.

    Args:
        port (:py:obj:`int`): the port number of the server that listens on the target device. default to 15004.
        device (:py:obj:`Device`): :py:obj:`airtest.core.device.Device` instance provided by ``airtest``. leave the
         parameter default and the default device will be chosen. more details refer to ``airtest doc``
        options: see :py:class:`poco.pocofw.Poco`

    Examples:
        The simplest way to connect to a cocos2dx-lua game::

            from poco.drivers.std import StdPoco
            from airtest.core.api import connect_device

            # connect a device first, then initialize poco object
            device = connect_device('Android:///')
            poco = StdPoco(10054, device)

            # or use ip:port to initialize poco object
            poco = StdPoco(port=10054, ip='xx.xx.xx.xx')

            # now you can play with poco
            ui = poco('...')
            ui.click()
            ...

    """

    def __init__(self, port=DEFAULT_PORT, device=None, use_airtest_input=True, ip=None, **kwargs):
        if ip is None or ip == "localhost":
            self.device = device or default_device()

            platform_name = device_platform(self.device)
            if platform_name == 'Android':
                # always forward for android device to avoid network unreachable
                local_port, _ = self.device.adb.setup_forward('tcp:{}'.format(port))
                ip = self.device.adb.host or 'localhost'
                port = local_port
            elif platform_name == 'IOS':
                port, _ = self.device.setup_forward(port)
                if self.device.is_local_device:
                    ip = 'localhost'
                else:
                    ip = self.device.ip
            else:
                try:
                    ip = self.device.get_ip_address()
                except AttributeError:
                    try:
                        ip = socket.gethostbyname(socket.gethostname())
                    except socket.gaierror:
                        # 某些特殊情况下会出现这个error，无法正确获取本机ip地址
                        ip = 'localhost'

        agent = StdPocoAgent((ip, port), use_airtest_input)
        kwargs['reevaluate_volatile_attributes'] = True
        super(StdPoco, self).__init__(agent, **kwargs)



from .utils import JSONSerializable


class JSONRPCBaseRequest(JSONSerializable):

    """ Base class for JSON-RPC 1.0 and JSON-RPC 2.0 requests."""

    def __init__(self, method=None, params=None, _id=None,
                 is_notification=None):
        self.data = dict()
        self.method = method
        self.params = params
        self._id = _id
        self.is_notification = is_notification

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, value):
        if not isinstance(value, dict):
            raise ValueError("data should be dict")

        self._data = value

    @property
    def args(self):
        """ Method position arguments.

        :return tuple args: method position arguments.

        """
        return tuple(self.params) if isinstance(self.params, list) else ()

    @property
    def kwargs(self):
        """ Method named arguments.

        :return dict kwargs: method named arguments.

        """
        return self.params if isinstance(self.params, dict) else {}

    @property
    def json(self):
        return self.serialize(self.data)


class JSONRPCBaseResponse(JSONSerializable):

    """ Base class for JSON-RPC 1.0 and JSON-RPC 2.0 responses."""

    def __init__(self, **kwargs):
        self.data = dict()

        try:
            self.result = kwargs['result']
        except KeyError:
            pass

        try:
            self.error = kwargs['error']
        except KeyError:
            pass

        self._id = kwargs.get('_id')

        if 'result' not in kwargs and 'error' not in kwargs:
            raise ValueError("Either result or error should be used")

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, value):
        if not isinstance(value, dict):
            raise ValueError("data should be dict")

        self._data = value

    @property
    def json(self):
        return self.serialize(self.data)



# -*- encoding=utf8 -*-
import re

from poco.exceptions import PocoTargetTimeout

from airtestProject.commons.UWA import *
from airtestProject.manager.LogManager import LogManager


class Utils:

    def __init__(self, package_name, poco, project):
        self.log_manager = LogManager(logfile=None,project=project)

        # todo 实例化poco前需要确保游戏已经稳定启动
        self.log_manager.log_step(f"实例化Utils{str(type(poco))}")
        self.package_name = package_name
        self.poco = poco

    def uwa_connect(self, modle, projectId):
        """
        连接uwa
        :param modle: uwa的录制模式
        :param projectId: 项目ID，可以从uwa项目中查看
        :return:TRUE/False
        """
        time.sleep(20)
        self.log_manager.log_subprocess('uwa_dump', modle, projectId)
        if modle is not None and modle != 'default' and projectId is not None:
            self.log_manager.log_step("GOT测试=》connect")
            GOT_Test.Connect(self.poco)
            self.log_manager.log_step("GOT测试=》start=》modle={}".format(modle))
            GOT_Test.Start(self.poco, modle)
            return True
        return False

    def uwa_dump(self, dumpType):
        """
        手动 dump，采集更多信息
        :param poco: poco实例对象
        :param dumpType: dump的类型，可用dump类型：mono/lua/resources/overdraw，类型为String
        :Example:
            >>> Utils.uwa_dump('overdraw')
        """
        dump = ['mono', 'lua', 'resources', 'overdraw']
        self.log_manager.log_subprocess('uwa_dump', dumpType)
        if dumpType not in dump:
            self.log_manager.log_step("执行GOT_Test.Dump方法{}".format(dumpType))
            GOT_Test.Dump(self.poco, dumpType)
            return True
        self.log_manager.log_error("执行uwa_dump方法，参数错误{}".format(dumpType))
        return False

    def uwa_tag(self, tag):
        """
        打tag标签
        :param poco: poco实例对象
        :param tag:区间标记
        :return:
        """
        self.log_manager.log_step("执行GOT_Test.Tag方法{}".format(tag))
        GOT_Test.Tag(self.poco, tag)

    def uwa_local_upload(self, account, password, timeLimits=3600, uploadTimeLimits=600):
        """
        上传本地uwa数据
        :param poco: poco实例化对象
        :param account: uwa账户名
        :param password: uwa账户密码
        :param timeLimits:测试时长限制，默认3600秒（1个小时）
        :param uploadTimeLimits:数据上传超时限制600秒
        :return:
        """
        self.log_manager.log_subprocess('uwa_local_upload', account, password, timeLimits, uploadTimeLimits)
        self.log_manager.log_step("执行GOT_Test.Stop方法")
        GOT_Test.Stop(self.poco)
        self.log_manager.log_step('执行GOT_Test.LocalUpload方法{}'.format(account, password, timeLimits, uploadTimeLimits))
        GOT_Test.LocalUpload(self.poco, account, password, timeLimits, uploadTimeLimits)
        self.log_manager.log_step('上传成功！执行stop_app方法{}'.format(self.package_name))
        stop_app(self.package_name)

    @logwrap
    def poco_click(self, node, index=0):
        """
        节点点击
        :param index: 是否有多个相同的node节点，如果有则填下标：
        eg:
        airtest中抓到node节点有3个相同的节点,需要获取第二个节点
        item_button
        item_button
        item_button
        >>> Utils.poco_click('item_button',1)
        :param node:poco节点
        :param index:相同节点的下标，从0开始
        :return:
        """
        pattern = r"image\d+\.png"
        match = re.search(pattern, node)
        if match:
            if exists(Template(node)):
                touch(Template(node))
            else:
                self.log_manager.log_step("{} 不存在，点击失败!".format(node))
        else:
            try:
                if '=' in node:
                    self.log_manager.log_step('节点包含=号：' + node)
                    poco_key = node.split('=')[0].strip()
                    poco_value = node.split('=')[1].strip().strip('"')
                    attr_dict = {poco_key: poco_value}
                    if not self.poco(attr_dict).exists():
                        return
                    self.log_manager.log_step('点击{}:{}节点'.format_map(attr_dict))
                    self.poco(**attr_dict).click()
                    return
                else:
                    if not self.poco(node).exists():
                        self.log_manager.log_error("节点{}不存在".format(node))
                        return
                    if index == 0:
                        self.log_manager.log_step('点击无相同元素的节点{}'.format(node))
                        # poco(node).get_position()得到的数据是[x,y]数组
                        # click(tuple(self.poco(node).get_position()))
                        self.poco(node).click(self.poco(node).get_position())
                        return
                    self.log_manager.log_step('点击有多个相同元素的节点,点击第{}个{}节点'.format(index, node))
                    self.poco(node)[index].click()
                    return
            except:
                self.log_manager.log_error("poco_click方法执行失败，节点为{}".format(node))

    def poco_long_click(self, node, seconds=3):
        """
        # 存在就点击
        :param node: 点击节点
        :param seconds: 长按时长(秒)
        :return:
        """
        try:
            self.log_manager.log_subprocess('poco_long_click', node, seconds)
            cur_node = self.poco(node)
            cur_node.long_click(duration=seconds)
        except:
            self.log_manager.log_error("执行poco_long_click方法错误，节点{}".format(node))

    def poco_swipe(self, node, direction=0, offset=0.3, duration=0.5):
        """
        滑动&长按
        :param duration: 持续时间，默认0.5,如果是长按
        :param direction: 滑动方向：0,1,2,3分别表示水平向右，水平向左，垂直向下，垂直向上
        :param node: uwa节点
        :param offset: 偏移量
        :return:
        """
        self.log_manager.log_subprocess('poco_swipe', node, direction, offset, duration)
        _pos = self.poco(node).get_position()
        # 正反方向水平滑动
        forward_pos = (_pos[0] + offset, 0)
        opposite_pos = (_pos[0] - offset, 0)
        # 上下方向垂直滑动
        down_pos = (0, _pos[1] - offset)
        up_pos = (0, _pos[1] + offset)
        other_pos = (_pos[0] + offset, _pos[1] + offset)
        if direction == 0:
            # 水平向右
            self.log_manager.log_step('向右滑动节点{}'.format(node))
            swipe(_pos, vector=forward_pos, duration=duration)
        elif direction == 1:
            self.log_manager.log_step('向左滑动节点{}'.format(node))
            swipe(_pos, vector=opposite_pos, duration=duration)
        elif direction == 2:
            self.log_manager.log_step('向下滑动节点{}'.format(node))
            swipe(_pos, vector=down_pos, duration=duration)
        elif direction == 3:
            self.log_manager.log_step('向上滑动节点{}'.format(node))
            swipe(_pos, vector=up_pos, duration=duration)
        else:
            self.log_manager.log_step('其他方向滑动节点{}'.format(node))
            swipe(_pos, vector=other_pos, duration=duration)

    def node_wait_for_appearance(self, target_node, timeout=120):
        """
        等待元素出现
        :param poco: poco实例化
        :param target_node: 目标节点
        :param timeout: 持续时间
        :return:
        """
        self.log_manager.log_subprocess("node_wait_for_appearance", target_node, timeout)
        if '=' in target_node:
            self.log_manager.log_step('节点包含=号：' + target_node)
            poco_key = target_node.split('=')[0].strip()
            poco_value = target_node.split('=')[1].strip().strip('=')
            attr_dict = {poco_key: poco_value}
            start = time.time()
            log(attr_dict)
            temp_node = self.poco(**attr_dict)
            while not temp_node.exists():
                time.sleep(2)
                temp_node = self.poco(**attr_dict)
                log(temp_node)
                if time.time() - start > timeout:
                    self.log_manager.log_error('等待超时，节点{}'.format(target_node))
                    raise PocoTargetTimeout('appearance', self)
            position = temp_node.get_position()
            if position[0] > 1 or position[1] > 1:
                # 如果坐标位置超过1，则说明元素在屏幕外，可以忽略
                self.log_manager.log_error('{}节点坐标位置在屏幕外'.format(temp_node))
                return
            return temp_node
        else:
            start = time.time()
            while not self.poco(target_node).exists():
                self.log_manager.log_step('等待元素{}出现'.format(str(self.poco(target_node).exists())))
                # position = self.poco(**attr_dict).get_position
                time.sleep(2)
                if time.time() - start > timeout:
                    self.log_manager.log_error('等待超时，节点{}'.format(target_node))
                    raise PocoTargetTimeout('disappearance', self)
            return self.poco(target_node)

    def node_wait_for_disappearance(self, target_node, timeout=120):
        """
        # 等待元素消失
        :param poco:poco实例化
        :param target_node: 目标节点
        :param timeout: 超时时间
        :return:
        """
        self.log_manager.log_subprocess('node_wait_for_disappearance', target_node, timeout)
        if '=' in target_node:
            poco_key = target_node.split('=')[0].strip()
            poco_value = target_node.split('=')[1].strip().strip('=')
            attr_dict = {poco_key: poco_value}
            start = time.time()
            while self.poco(**attr_dict).exists():
                # position = self.poco(**attr_dict).get_position
                time.sleep(2)
                if time.time() - start > timeout:
                    self.log_manager.log_error('等待元素消失步骤，等待超时{}:{}'.format_map(attr_dict))
                    raise PocoTargetTimeout('disappearance', self)
            # if position[0] > 1 or position[1] > 1:
            #     # 如果坐标位置超过1，则说明元素在屏幕外，可以忽略
            #     return True
        else:
            start = time.time()
            while self.poco(target_node).exists():
                # position = self.poco(**attr_dict).get_position
                time.sleep(2)
                if time.time() - start > timeout:
                    self.log_manager.log_error('等待元素消失步骤，等待超时{}:{}'.format_map(target_node))
                    raise PocoTargetTimeout('disappearance', self)
        return True

    def poco_dialog(self, node):
        """
        剧情对话点击
        :param node:对话节点
        :return:
        """
        self.log_manager.log_subprocess('poco_dialog', node)
        dialog_node = self.poco(node)
        # todo 这里可能出现判断失败的情况，需要后续观察
        while dialog_node.exists():
            self.log_manager.log_step('对话节点存在{}'.format(node))
            self.log_manager.log_step('点击对话节点{}'.format(node))
            self.poco_click(node)
            sleep(2)
        # return

    def poco_battle(self, skill_node, stop_node):
        """
        战斗逻辑：进入战斗后
        :param stop_node:停止的节点，用来判断是否需要停止战斗的节点
        :param skill_node:使用技能的节点
        :return:
        """
        self.log_manager.log_subprocess('poco_battle', skill_node, stop_node)
        use_skill = self.poco(skill_node)
        stop = self.poco(stop_node)
        while stop.exists():
            time.sleep(1)
            # 检查攻击目标UI是否存在，不存在可能怪物已死
            btn_search_and_lock = self.poco('btnSearchAndLock')
            if btn_search_and_lock:
                # 连击3次
                self.poco(use_skill)
                self.poco(use_skill)
                self.poco(use_skill)
                # 如果死亡了
                # btnReviveNormal =
            else:
                btn_search_and_lock.click()
                time.sleep(1)
                self.poco(use_skill)
                self.poco(use_skill)
                self.poco(use_skill)
            stop = self.poco(stop_node)
        stop.click()
        # return True

    def poco_use_skills(self, skill_nodes, task_node="", cur_task_name=""):
        """
        # 攻击
        :param skill_nodes: 
        :param cur_task_name: 
        :param task_node: 
        :param skill_node:技能节点列表([])
        :return:
        """

        log(skill_nodes)

        cur_task = self.poco(task_node)
        while cur_task.attr("text") == cur_task_name:
            btn_search_and_lock = self.poco('btnSearchAndLock')

            if not btn_search_and_lock:
                self.poco_click("attackSkill")
                self.poco_click('btnSearchAndLock')

            if len(skill_nodes) == 1:
                self.poco_long_click(skill_nodes[0], t=3)
            else:
                for i, node in enumerate(skill_nodes[1:]):
                    log("攻击技能", node)
                    self.poco_click(node)
                    self.poco_long_click(skill_nodes[0], t=3)
            cur_task = self.poco(task_node)
            print(cur_task.attr("text"), cur_task_name)


    def sleep(self, seconds=60):
        self.log_manager.log_step('睡眠{}秒'.format(seconds))
        time.sleep(seconds)



# This file is part of Androguard.
#
# Copyright (C) 2010, Anthony Desnos <desnos at t0t0.fr>
# All rights reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.

from . import typeconstants as tc
from .axmlparser import AXMLParser

from struct import pack, unpack
from xml.sax import saxutils


class AXMLPrinter:
    def __init__(self, raw_buff):
        self.axml = AXMLParser(raw_buff)
        self.xmlns = False

        self.buff = ""

        while 1:
            _type = self.axml.next()
            #print "tagtype = ", _type

            if _type == tc.START_DOCUMENT:
                self.buff += "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
            elif _type == tc.START_TAG:
                self.buff += "<%s%s\n" % (self.getPrefix(self.axml.getPrefix()), self.axml.getName())

                # FIXME: use namespace
                if self.xmlns == False:
                    self.buff += "xmlns:%s=\"%s\"\n" % (self.axml.getNamespacePrefix(0), self.axml.getNamespaceUri(0))
                    self.xmlns = True

                for i in range(0, self.axml.getAttributeCount()):
                    self.buff += "%s%s=\"%s\"\n" % (self.getPrefix(self.axml.getAttributePrefix(i)), self.axml.getAttributeName(i), self.getAttributeValue(i))

                self.buff += ">\n"

            elif _type == tc.END_TAG:
                self.buff += "</%s%s>\n" % (self.getPrefix(self.axml.getPrefix()), self.axml.getName())

            elif _type == tc.TEXT:
                self.buff += "%s\n" % self.axml.getText()

            elif _type == tc.END_DOCUMENT:
                break

    def getBuff(self):
        return self.buff.encode("utf-8")

    def getPrefix(self, prefix):
        if prefix == None or len(prefix) == 0:
            return ""

        return prefix + ":"

    def getAttributeValue(self, index):
        _type = self.axml.getAttributeValueType(index)
        _data = self.axml.getAttributeValueData(index)

        #print _type, _data
        if _type == tc.TYPE_STRING:
            return saxutils.escape(self.axml.getAttributeValue(index), entities={'"': '&quot;'})

        elif _type == tc.TYPE_ATTRIBUTE:
            return "?%s%08X" % (self.getPackage(_data), _data)

        elif _type == tc.TYPE_REFERENCE:
            return "@%s%08X" % (self.getPackage(_data), _data)

        # WIP
        elif _type == tc.TYPE_FLOAT:
            return "%f" % unpack("=f", pack("=L", _data))[0]

        elif _type == tc.TYPE_INT_HEX:
            return "0x%08X" % _data

        elif _type == tc.TYPE_INT_BOOLEAN:
            if _data == 0:
                return "false"
            return "true"

        elif _type == tc.TYPE_DIMENSION:
            return "%f%s" % (self.complexToFloat(_data), tc.DIMENSION_UNITS[_data & tc.COMPLEX_UNIT_MASK])

        elif _type == tc.TYPE_FRACTION:
            return "%f%s" % (self.complexToFloat(_data), tc.FRACTION_UNITS[_data & tc.COMPLEX_UNIT_MASK])

        elif _type >= tc.TYPE_FIRST_COLOR_INT and _type <= tc.TYPE_LAST_COLOR_INT:
            return "#%08X" % _data

        elif _type >= tc.TYPE_FIRST_INT and _type <= tc.TYPE_LAST_INT:
            if _data > 0x7fffffff:
                _data = (0x7fffffff & _data) - 0x80000000
                return "%d" % _data
            elif _type == tc.TYPE_INT_DEC:
                return "%d" % _data

        # raise exception here?
        return "<0x%X, type 0x%02X>" % (_data, _type)

    def complexToFloat(self, xcomplex):
        return (float)(xcomplex & 0xFFFFFF00) * tc.RADIX_MULTS[(xcomplex>>4) & 3];

    def getPackage(self, id):
        if id >> 24 == 1:
            return "android:"
        return ""




# coding=utf-8

from airtestProject.poco.drivers.unity3d.test.tutorial.case import TutorialCase


class LongClickTutorial(TutorialCase):
    def runTest(self):
        self.poco('btn_start').click()
        self.poco('basic').click()
        self.poco('star_single').long_click()
        self.poco('star_single').long_click(duration=5)


if __name__ == '__main__':
    import pocounit
    pocounit.main()



# coding=utf-8

from airtestProject.airtest.core.api import connect_device


def UE4EditorWindow():
    dev = connect_device("Windows:///?class_name=UnrealWindow&title_re=.*Game Preview Standalone.*")
    game_window = dev.app.top_window()
    dev._top_window = game_window.wrapper_object()
    dev.focus_rect = (5, 29, 5, 5)
    return dev



"""
This package provide IOS Device Class.
"""
import six
if six.PY2:
    raise ImportError(
        "The iOS module of Airtest>1.1.7 only supports Python3, if you want to use, please upgrade to Python3 first.")
from airtest.core.ios.ios import IOS, TIDevice



__version__ = "1.3.3.1"

import os
import sys


def get_airtest_version():
    pip_pkg_dir = os.path.join(os.path.dirname(__file__), "..", "..")
    pip_pkg_dir = os.path.abspath(pip_pkg_dir)

    return (
        'airtest {} from {} (python {})'.format(
            __version__, pip_pkg_dir, sys.version[:3],
        )
    )


def show_version():
    sys.stdout.write(get_airtest_version())
    sys.stdout.write(os.linesep)
    sys.exit()



# -*- coding: utf-8 -*-
import os
import re
import socket
import sys
import warnings

from airtestProject.airtest.core.android.constant import STFLIB
from airtestProject.airtest.core.android.touch_methods.base_touch import BaseTouch
from airtestProject.airtest.utils.logger import get_logger
from airtestProject.airtest.utils.nbsp import NonBlockingStreamReader
from airtestProject.airtest.utils.safesocket import SafeSocket
from airtestProject.airtest.utils.snippet import get_std_encoding, kill_proc, reg_cleanup

LOGGING = get_logger(__name__)


class Minitouch(BaseTouch):

    def __init__(self, adb, backend=False, size_info=None, input_event=None):
        super(Minitouch, self).__init__(adb, backend, size_info, input_event)
        self.default_pressure = 50
        self.path_in_android = "/data/local/tmp/minitouch"
        self.max_x, self.max_y = None, None
        self.localport = None

    def install(self):
        """
        Install minitouch

        Returns:
            None

        """

        abi = self.adb.getprop("ro.product.cpu.abi")
        sdk = int(self.adb.getprop("ro.build.version.sdk"))

        if sdk >= 16:
            binfile = "minitouch"
        else:
            binfile = "minitouch-nopie"

        device_dir = os.path.dirname(self.path_in_android)
        path = os.path.join(STFLIB, abi, binfile).replace("\\", r"\\")

        try:
            exists_file = self.adb.file_size(self.path_in_android)
        except:
            pass
        else:
            local_minitouch_size = int(os.path.getsize(path))
            if exists_file and exists_file == local_minitouch_size:
                LOGGING.debug("install_minitouch skipped")
                return
            self.uninstall()

        self.adb.push(path, "%s/minitouch" % device_dir)
        self.adb.shell("chmod 755 %s/minitouch" % (device_dir))
        LOGGING.info("install_minitouch finished")

    def uninstall(self):
        """
        Uninstall minitouch

        Returns:
            None

        """
        self.adb.raw_shell("rm " + self.path_in_android)

    def setup_server(self):
        """
        Setup minitouch server and adb forward

        Returns:
            server process

        """
        if self.server_proc:
            self.server_proc.kill()
            self.server_proc = None

        self.localport, deviceport = self.adb.setup_forward("localabstract:minitouch_{}".format)
        deviceport = deviceport[len("localabstract:"):]
        if self.input_event:
            p = self.adb.start_shell("/data/local/tmp/minitouch -n '{0}' -d '{1}' 2>&1".format(deviceport,self.input_event))
        else:
            p = self.adb.start_shell("/data/local/tmp/minitouch -n '{0}' 2>&1".format(deviceport))
        nbsp = NonBlockingStreamReader(p.stdout, name="minitouch_server", auto_kill=True)
        while True:
            line = nbsp.readline(timeout=3.0)
            if line is None:
                kill_proc(p)
                raise RuntimeError("minitouch setup timeout")

            line = line.decode(get_std_encoding(sys.stdout))

            # 识别出setup成功的log，并匹配出max_x, max_y
            m = re.search("Type \w touch device .+ \((\d+)x(\d+) with \d+ contacts\) detected on .+ \(.+\)", line)
            if m:
                self.max_x, self.max_y = int(m.group(1)), int(m.group(2))
                break
            else:
                self.max_x = 32768
                self.max_y = 32768
        if p.poll() is not None:
            # server setup error, may be already setup by others
            # subprocess exit immediately
            kill_proc(p)
            raise RuntimeError("minitouch server quit immediately")
        self.server_proc = p
        reg_cleanup(kill_proc, self.server_proc)
        return p

    def setup_client(self):
        """
        Setup client in following steps::

            1. connect to server
            2. receive the header
                v <version>
                ^ <max-contacts> <max-x> <max-y> <max-pressure>
                $ <pid>
            3. prepare to send

        Returns:
            None

        """
        s = SafeSocket()
        s.connect((self.adb.host, self.localport))
        s.sock.settimeout(2)
        header = b""
        while True:
            try:
                header += s.sock.recv(4096)  # size is not strict, so use raw socket.recv
            except socket.timeout:
                # raise RuntimeError("minitouch setup client error")
                warnings.warn("minitouch header not recved")
                break
            if header.count(b'\n') >= 3:
                break
        LOGGING.debug("minitouch header:%s", repr(header))
        self.client = s
        self.handle = self.safe_send

    def transform_xy(self, x, y):
        """
        Transform coordinates (x, y) according to the device display

        Args:
            x: coordinate x
            y: coordinate y

        Returns:
            transformed coordinates (x, y)

        """
        width, height = self.size_info['width'], self.size_info['height']
        nx = float(x) * self.max_x / width
        ny = float(y) * self.max_y / height
        return "%.0f" % nx, "%.0f" % ny

    def teardown(self):
        super(Minitouch, self).teardown()
        if self.localport:
            self.adb.remove_forward("tcp:{}".format(self.localport))
            self.localport = None



# -*- coding: utf-8 -*-
import os
import re
import traceback
import struct
import threading
import six
import socket
from functools import wraps, partial
from airtestProject.airtest.core.android.constant import STFLIB
from airtestProject.airtest.utils.logger import get_logger
from airtestProject.airtest.utils.nbsp import NonBlockingStreamReader
from airtestProject.airtest.utils.safesocket import SafeSocket
from airtestProject.airtest.utils.snippet import reg_cleanup, on_method_ready, ready_method, kill_proc
from airtestProject.airtest.utils.threadsafe import threadsafe_generator
from airtestProject.airtest.core.android.cap_methods.base_cap import BaseCap
from airtestProject.airtest import aircv
from airtestProject.airtest.core.error import ScreenError

LOGGING = get_logger(__name__)


def retry_when_socket_error(func):
    @wraps(func)
    def wrapper(inst, *args, **kwargs):
        try:
            return func(inst, *args, **kwargs)
        except socket.error:
            inst.frame_gen = None
            return func(inst, *args, **kwargs)
    return wrapper


class Minicap(BaseCap):
    """super fast android screenshot method from stf minicap.

    reference https://github.com/openstf/minicap
    """

    VERSION = 5
    RECVTIMEOUT = 3  # default value is None, but the version above 1.2.7 is changed to 3s
    CMD = "LD_LIBRARY_PATH=/data/local/tmp /data/local/tmp/minicap"

    def __init__(self, adb, projection=None, rotation_watcher=None, display_id=None, ori_function=None):
        """
        :param adb: adb instance of android device
        :param projection: projection, default is None. If `None`, physical display size is used
        """
        super(Minicap, self).__init__(adb=adb)
        self.projection = projection
        self.display_id = display_id
        self.ori_function = ori_function or self.adb.get_display_info
        self.frame_gen = None
        self.stream_lock = threading.Lock()
        self.quirk_flag = 0
        self._stream_rotation = None
        self._update_rotation_event = threading.Event()
        if rotation_watcher:
            # Minicap needs to be reconnected when switching between landscape and portrait
            # minicap需要在横竖屏转换时，重新连接
            rotation_watcher.reg_callback(lambda x: self.update_rotation(x * 90))
        self.cleanup_func = []
        # Force cleanup on exit
        reg_cleanup(self.teardown_stream)

    @ready_method
    def install_or_upgrade(self):
        """
        Install or upgrade minicap

        Returns:
            None

        """
        if self.adb.exists_file("/data/local/tmp/minicap") \
                and self.adb.exists_file("/data/local/tmp/minicap.so"):
            try:
                output = self.adb.raw_shell("%s -v 2>&1" % self.CMD)
            except Exception as err:
                LOGGING.error(str(err))
                version = -1
            else:
                LOGGING.debug(output.strip())
                m = re.match("version:(\d)", output)
                if m:
                    version = int(m.group(1))
                else:
                    version = -1
            if version >= self.VERSION:
                LOGGING.debug('skip install minicap')
                return
            else:
                LOGGING.debug('upgrade minicap to lastest version: %s->%s' % (version, self.VERSION))
                self.uninstall()
        else:
            LOGGING.debug('install minicap')
        self.install()

    def uninstall(self):
        """
        Uninstall minicap

        Returns:
            None

        """
        try:
            self.adb.raw_shell("rm -r /data/local/tmp/minicap*")
        except Exception as e:
            # AdbError: No such file or directory
            LOGGING.warning(e)

    def install(self):
        """
        Install minicap

        Reference: https://github.com/openstf/minicap/blob/master/run.sh

        Returns:
            None

        """
        abi = self.adb.getprop("ro.product.cpu.abi")
        pre = self.adb.getprop("ro.build.version.preview_sdk")
        rel = self.adb.getprop("ro.build.version.release")
        sdk = self.adb.sdk_version

        if pre.isdigit() and int(pre) > 0:
            sdk += 1

        if sdk >= 16:
            binfile = "minicap"
        else:
            binfile = "minicap-nopie"

        device_dir = "/data/local/tmp"

        path = os.path.join(STFLIB, abi, binfile)
        self.adb.push(path, "%s/minicap" % device_dir)
        self.adb.shell("chmod 755 %s/minicap" % device_dir)

        pattern = os.path.join(STFLIB, 'minicap-shared/aosp/libs/android-%s/%s/minicap.so')
        path = pattern % (sdk, abi)
        if not os.path.isfile(path):
            path = pattern % (rel, abi)

        self.adb.push(path, "%s/minicap.so" % device_dir)
        self.adb.shell("chmod 755 %s/minicap.so" % device_dir)
        LOGGING.info("minicap installation finished")

    @on_method_ready('install_or_upgrade')
    def get_frame(self, projection=None):
        """
        Get the single frame from minicap -s, this method slower than `get_frames`
            1. shell cmd
            1. remove log info
            1. \r\r\n -> \n ...

        Args:
            projection: screenshot projection, default is None which means using self.projection

        Returns:
            jpg data

        """
        params, display_info = self._get_params(projection)
        if self.display_id:
            raw_data = self.adb.raw_shell(
                self.CMD + " -d " + str(self.display_id) + " -n 'airtest_minicap' -P %dx%d@%dx%d/%d -s" % params,
                ensure_unicode=False,
            )
        else:
            raw_data = self.adb.raw_shell(
                self.CMD + " -n 'airtest_minicap' -P %dx%d@%dx%d/%d -s" % params,
                ensure_unicode=False,
            )
        jpg_data = raw_data.split(b"for JPG encoder" + self.adb.line_breaker)[-1]
        jpg_data = jpg_data.replace(self.adb.line_breaker, b"\n")
        if jpg_data.startswith(b"\xff\xd8") and jpg_data.endswith(b"\xff\xd9"):
            return jpg_data
        else:
            raise ScreenError("invalid jpg format")

    def _get_params(self, projection=None):
        """
        Get the minicap origin parameters and count the projection

        Returns:
            physical display size (width, height), counted projection (width, height) and real display orientation

        """
        display_info = self.ori_function()
        real_width = display_info["width"]
        real_height = display_info["height"]
        real_rotation = display_info["rotation"]
        # 优先去传入的projection
        projection = projection or self.projection
        if projection:
            proj_width, proj_height = projection
        else:
            proj_width, proj_height = real_width, real_height

        if self.quirk_flag & 2 and real_rotation in (90, 270):
            params = real_height, real_width, proj_height, proj_width, 0
        else:
            params = real_width, real_height, proj_width, proj_height, real_rotation

        return (params, display_info)

    @on_method_ready('install_or_upgrade')
    def get_stream(self, lazy=True):
        """
        Get stream, it uses `adb forward`and socket communication. Use minicap ``lazy``mode (provided by gzmaruijie)
        for long connections - returns one latest frame from the server


        Args:
            lazy: True or False

        Returns:

        """
        gen = self._get_stream(lazy)

        # if quirk error, restart server and client once
        stopped = next(gen)

        if stopped:
            try:
                next(gen)
            except StopIteration:
                pass
            gen = self._get_stream(lazy)
            next(gen)

        return gen

    @threadsafe_generator
    @on_method_ready('install_or_upgrade')
    def _get_stream(self, lazy=True):
        self._cleanup_minicap()
        proc, nbsp, localport = self._setup_stream_server(lazy=lazy)
        s = SafeSocket()
        s.connect((self.adb.host, localport))
        t = s.recv(24)
        # minicap header
        global_headers = struct.unpack("<2B5I2B", t)
        LOGGING.debug(global_headers)
        # check quirk-bitflags, reference: https://github.com/openstf/minicap#quirk-bitflags
        ori, self.quirk_flag = global_headers[-2:]

        if self.quirk_flag & 2 and ori in (1, 3):
            # resetup
            LOGGING.debug("quirk_flag found, going to resetup")
            stopping = True
        else:
            stopping = False
        self.cleanup_func.append(s.close)
        self.cleanup_func.append(nbsp.kill)
        self.cleanup_func.append(partial(kill_proc, proc))
        self.cleanup_func.append(partial(self.adb.remove_forward, "tcp:%s" % localport))
        yield stopping

        while not stopping:
            if lazy:
                s.send(b"1")
            # recv frame header, count frame_size
            if self.RECVTIMEOUT is not None:
                # Some mobile phones may keep waiting for data when switching between horizontal and vertical screens,
                # and the connection is not closed, resulting in a black screen
                # Set the timeout to 3s(airtest>=1.2.7)
                header = s.recv_with_timeout(4, self.RECVTIMEOUT)
            else:
                header = s.recv(4)
            if header is None:
                LOGGING.error("minicap header is None")
                # recv timeout, if not frame updated, maybe screen locked
                stopping = yield None
            else:
                frame_size = struct.unpack("<I", header)[0]
                if self.RECVTIMEOUT is not None:
                    frame_data = s.recv_with_timeout(frame_size, self.RECVTIMEOUT)
                else:
                    frame_data = s.recv(frame_size)
                stopping = yield frame_data

        LOGGING.debug("minicap stream ends")
        # teardown stream() cannot be called directly because the connection may be rebuilt multiple times
        # while the screen is rotated, and self.frame_gen gets stuck
        self._cleanup()

    def _setup_stream_server(self, lazy=False):
        """
        Setup minicap process on device

        Args:
            lazy: parameter `-l` is used when True

        Returns:
            adb shell process, non-blocking stream reader and local port

        """
        localport, deviceport = self.adb.setup_forward("localabstract:minicap_{}".format)
        deviceport = deviceport[len("localabstract:"):]
        other_opt = "-l" if lazy else ""
        params, display_info = self._get_params()
        if self.display_id:
            proc = self.adb.start_shell(
                "%s -d %s -n '%s' -P %dx%d@%dx%d/%d %s 2>&1" %
                tuple([self.CMD, self.display_id, deviceport] + list(params) + [other_opt]),
            )
        else:
            proc = self.adb.start_shell(
                "%s -n '%s' -P %dx%d@%dx%d/%d %s 2>&1" %
                tuple([self.CMD, deviceport] + list(params) + [other_opt]),
            )
        nbsp = NonBlockingStreamReader(proc.stdout, print_output=True, name="minicap_server", auto_kill=True)
        while True:
            line = nbsp.readline(timeout=5.0)
            if line is None:
                kill_proc(proc)
                raise RuntimeError("minicap server setup timeout")
            if b"Server start" in line:
                break

        if proc.poll() is not None:
            # minicap server setup error, may be already setup by others
            # subprocess exit immediately
            kill_proc(proc)
            raise RuntimeError("minicap server quit immediately")

        self._stream_rotation = int(display_info["rotation"])
        return proc, nbsp, localport

    @retry_when_socket_error
    def get_frame_from_stream(self):
        """
        Get one frame from minicap stream

        Returns:
            frame

        """
        if self._update_rotation_event.is_set():
            LOGGING.debug("do update rotation")
            self.teardown_stream()
            self._update_rotation_event.clear()
        if self.frame_gen is None:
            self.frame_gen = self.get_stream()
        return six.next(self.frame_gen)

    def snapshot(self, ensure_orientation=True, projection=None):
        """

        Args:
            ensure_orientation: True or False whether to keep the orientation same as display
            projection: the size of the desired projection, (width, height)

        Returns:

        """
        if projection:
            # minicap模式在单张截图时，可以传入projection参数来强制指定图片大小，如手机分辨率(width, height)
            screen = self.get_frame(projection=projection)
            try:
                screen = aircv.utils.string_2_img(screen)
            except Exception:
                # may be black/locked screen or other reason, print exc for debugging
                traceback.print_exc()
                return None
            return screen
        else:
            return super(Minicap, self).snapshot()

    def update_rotation(self, rotation):
        """
        Update rotation and reset the backend stream generator

        Args:
            rotation: rotation input

        Returns:
            None

        """
        LOGGING.debug("update_rotation: %s" % rotation)
        self._update_rotation_event.set()

    def _cleanup_minicap(self):
        """
        Clean up the minicap process whose status is __skb_wait_for_more_packets or futex_wait_queue_me
        清理状态为__skb_wait_for_more_packets, futex_wait_queue_me的minicap进程

        Returns:

        """
        # 卡住的进程状态
        TASK_INTERRUPTIBLE1 = "__skb_wait_for_more_packets"
        TASK_INTERRUPTIBLE2 = "futex_wait_queue_me"

        shell_output = ""
        try:
            shell_output = self.adb.shell("ps -A| grep minicap")
        except:
            try:
                shell_output = self.adb.shell("ps| grep minicap")
            except:
                pass

        if len(shell_output) == 0:
            return
        for line in shell_output.split("\r\n"):
            if TASK_INTERRUPTIBLE1 in line or TASK_INTERRUPTIBLE2 in line:
                pid = line.split()[1]
                try:
                    self.adb.shell("kill %s" % pid)
                except:
                    pass

    def _cleanup(self):
        """
        Cleanup minicap process and stream reader

        主动将minicap建立的各个连接关闭
        与snippet.py中的CLEANUP_CALLS功能相同，但是允许主动调用，避免异常退出时有遗漏进程没清理干净

        Returns:

        """
        for func in self.cleanup_func:
            func()
        self.cleanup_func = []

    def teardown_stream(self):
        """
        End the stream

        Returns:
            None

        """
        # clean up established connections
        self._cleanup()
        if not self.frame_gen:
            return
        try:
            self.frame_gen.send(1)
        except (TypeError, StopIteration):
            # TypeError: can't send non-None value to a just-started generator
            pass
        else:
            LOGGING.warn("%s tear down failed" % self.frame_gen)
        self.frame_gen = None



# -*- coding: utf-8 -*-
import argparse
import sys
from airtestProject.airtest.report.report import get_parger as report_parser
from airtestProject.airtest.cli.runner import setup_by_args


def get_parser():
    ap = argparse.ArgumentParser()
    subparsers = ap.add_subparsers(dest="action", help="version/run/info/report")
    # subparser version
    subparsers.add_parser("version", help="show version and exit")
    # subparser run
    ap_run = subparsers.add_parser("run", help="run script")
    runner_parser(ap_run)
    # subparser info
    ap_info = subparsers.add_parser("info", help="get & print author/title/desc info of script")
    ap_info.add_argument("script", help="script filename")
    # subparser report
    ap_report = subparsers.add_parser("report", help="generate report of script")
    report_parser(ap_report)
    return ap


def runner_parser(ap=None):
    if not ap:
        ap = argparse.ArgumentParser()
    ap.add_argument("script", help="air path")
    ap.add_argument("--device", help="connect dev by uri string, e.g. Android:///", nargs="?", action="append")
    ap.add_argument("--log", help="set log dir, default to be script dir", nargs="?", const=True)
    ap.add_argument("--compress", required=False, type=int, choices=range(1, 100), help="set snapshot quality, 1-99", default=10)
    ap.add_argument("--recording", help="record screen when running", nargs="?", const=True)
    ap.add_argument("--no-image", help="Do not save screenshots", nargs="?", const=True)
    return ap


def cli_setup(args=None):
    """future api for setup env by cli"""
    if not args:
        if len(sys.argv) < 2:
            print("no cmdline args")
            return False
        args = sys.argv
    print(args)

    ap = argparse.ArgumentParser()
    if "--report" in args:
        from airtest.report.report import main as report_main
        ap = report_parser(ap)
        args = ap.parse_args(args)
        report_main(args)
        exit(0)
    else:
        ap = runner_parser(ap)
        args = ap.parse_args(args)
        setup_by_args(args)
    return True



# encoding=utf-8
import warnings
from collections import OrderedDict
from airtestProject.airtest.core.android.touch_methods.minitouch import Minitouch
from airtestProject.airtest.core.android.touch_methods.maxtouch import Maxtouch
from airtestProject.airtest.core.android.touch_methods.base_touch import BaseTouch
from airtestProject.airtest.core.android.constant import TOUCH_METHOD
from airtestProject.airtest.utils.logger import get_logger


LOGGING = get_logger(__name__)


class TouchProxy(object):
    """
    Perform touch operation according to the specified method
    """
    TOUCH_METHODS = OrderedDict()

    def __init__(self, touch_method):
        self.touch_method = touch_method

    def __getattr__(self, name):
        if name == "method_name":
            return self.touch_method.METHOD_NAME
        method = getattr(self.touch_method, name, getattr(self.touch_method.base_touch, name, None))
        if method:
            return method
        else:
            raise NotImplementedError("%s does not support %s method" %
                                      (getattr(self.touch_method, "METHOD_NAME", ""), name))

    @classmethod
    def check_touch(cls, touch_impl):
        try:
            touch_impl.base_touch.install_and_setup()
        except Exception as e:
            LOGGING.error(e)
            LOGGING.warning("%s setup up failed!" % touch_impl.METHOD_NAME)
            return False
        else:
            return True

    @classmethod
    def auto_setup(cls, adb, default_method=None, ori_transformer=None, size_info=None, input_event=None):
        """

        Args:
            adb: :py:mod:`airtest.core.android.adb.ADB`
            default_method: The default click method, such as "MINITOUCH"
            ori_transformer: dev._touch_point_by_orientation
            size_info: the result of dev.get_display_info()
            input_event: dev.input_event
            *args:
            **kwargs:

        Returns: TouchProxy object

        Examples:
            >>> dev = Android()
            >>> touch_proxy = TouchProxy.auto_setup(dev.adb, ori_transformer=dev._touch_point_by_orientation)
            >>> touch_proxy.touch((100, 100))

        """
        if default_method and default_method in cls.TOUCH_METHODS:
            touch_method = cls.TOUCH_METHODS[default_method].METHOD_CLASS(adb, size_info=size_info,
                                                                          input_event=input_event)
            impl = cls.TOUCH_METHODS[default_method](touch_method, ori_transformer)
            if cls.check_touch(impl):
                return TouchProxy(impl)

        if not default_method:
            for name, touch_impl in cls.TOUCH_METHODS.items():
                touch_method = touch_impl.METHOD_CLASS(adb, size_info=size_info, input_event=input_event)
                impl = touch_impl(touch_method, ori_transformer)
                if cls.check_touch(impl):
                    return TouchProxy(impl)

        # If both minitouch and maxtouch fail to initialize, use adbtouch
        # 如果minitouch和maxtouch都初始化失败，使用adbtouch
        adb_touch = AdbTouchImplementation(adb)
        warnings.warn("Currently using ADB touch, the efficiency may be very low.")
        return TouchProxy(adb_touch)


def register_touch(cls):
    TouchProxy.TOUCH_METHODS[cls.METHOD_NAME] = cls
    return cls


class AdbTouchImplementation(object):
    METHOD_NAME = TOUCH_METHOD.ADBTOUCH

    def __init__(self, base_touch):
        """

        :param base_touch: :py:mod:`airtest.core.android.adb.ADB`
        """
        self.base_touch = base_touch

    def touch(self, pos, duration=0.01):
        if duration <= 0.01:
            self.base_touch.touch(pos)
        else:
            self.swipe(pos, pos, duration=duration)

    def swipe(self, p1, p2, duration=0.5, *args, **kwargs):
        duration *= 1000
        self.base_touch.swipe(p1, p2, duration=duration)

    def teardown(self):
        pass


@register_touch
class MinitouchImplementation(AdbTouchImplementation):
    METHOD_NAME = TOUCH_METHOD.MINITOUCH
    METHOD_CLASS = Minitouch

    def __init__(self, minitouch, ori_transformer):
        """

        :param minitouch: :py:mod:`airtest.core.android.touch_methods.minitouch.Minitouch`
        :param ori_transformer: Android._touch_point_by_orientation()
        """
        super(MinitouchImplementation, self).__init__(minitouch)
        self.ori_transformer = ori_transformer

    def touch(self, pos, duration=0.01):
        pos = self.ori_transformer(pos)
        self.base_touch.touch(pos, duration=duration)

    def swipe(self, p1, p2, duration=0.5, steps=5, fingers=1):
        p1 = self.ori_transformer(p1)
        p2 = self.ori_transformer(p2)
        if fingers == 1:
            self.base_touch.swipe(p1, p2, duration=duration, steps=steps)
        elif fingers == 2:
            self.base_touch.two_finger_swipe(p1, p2, duration=duration, steps=steps)
        else:
            raise Exception("param fingers should be 1 or 2")

    def pinch(self, center=None, percent=0.5, duration=0.5, steps=5, in_or_out='in'):
        if center:
            center = self.ori_transformer(center)
        self.base_touch.pinch(center=center, percent=percent, duration=duration, steps=steps, in_or_out=in_or_out)

    def swipe_along(self, coordinates_list, duration=0.8, steps=5):
        pos_list = [self.ori_transformer(xy) for xy in coordinates_list]
        self.base_touch.swipe_along(pos_list, duration=duration, steps=steps)

    def two_finger_swipe(self, tuple_from_xy, tuple_to_xy, duration=0.8, steps=5, offset=(0, 50)):
        tuple_from_xy = self.ori_transformer(tuple_from_xy)
        tuple_to_xy = self.ori_transformer(tuple_to_xy)
        self.base_touch.two_finger_swipe(tuple_from_xy, tuple_to_xy, duration=duration, steps=steps, offset=offset)

    def perform(self, motion_events, interval=0.01):
        self.base_touch.perform(motion_events, interval)


@register_touch
class MaxtouchImplementation(MinitouchImplementation):
    METHOD_NAME = TOUCH_METHOD.MAXTOUCH
    METHOD_CLASS = Maxtouch

    def __init__(self, maxtouch, ori_transformer):
        """
        New screen click scheme, support Android10
        新的屏幕点击方案，支持Android10以上版本

        :param maxtouch: :py:mod:`airtest.core.android.touch_methods.maxtouch.Maxtouch`
        :param ori_transformer: Android._touch_point_by_orientation()
        """
        super(MaxtouchImplementation, self).__init__(maxtouch, ori_transformer)

    def perform(self, motion_events, interval=0.01):
        self.base_touch.perform(motion_events, interval)



# This file is part of Androguard.
#
# Copyright (C) 2010, Anthony Desnos <desnos at t0t0.fr>
# All rights reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.


from .axmlprinter import AXMLPrinter
import zipfile
from xml.dom import minidom


class APK:
    """APK manages apk file format"""
    def __init__(self, filename):
        """
            @param filename: specify the path of the file, or raw data
            @param raw: specify (boolean) if the filename is a path or raw data
        """
        self.filename = filename

        self.xml = {}
        self.package = ""
        self.androidversion = {}
        self._permissions = []
        self.valid_apk = False

        with open(filename, "rb") as fd:
            self.__raw = fd.read()

        self.zip = zipfile.ZipFile(filename)

        for i in self.zip.namelist():
            if i == "AndroidManifest.xml":
                self.xml[i] = minidom.parseString(AXMLPrinter(self.zip.read(i)).getBuff())

                self.package = self.xml[i].documentElement.getAttribute("package")
                self.androidversion["Code"] = self.xml[i].documentElement.getAttribute("android:versionCode")
                self.androidversion["Name"] = self.xml[i].documentElement.getAttribute("android:versionName")

                for item in self.xml[i].getElementsByTagName("uses-permission"):
                    self._permissions.append(str(item.getAttribute("android:name")))

                self.valid_apk = True

    def is_valid_apk(self):
        return self.valid_apk

    def get_filename(self):
        """
            Return the filename of the APK
        """
        return self.filename

    def get_package(self):
        """
            Return the name of the package
        """
        return self.package

    def get_androidversion_code(self):
        """
            Return the android version code
        """
        return self.androidversion["Code"]
    androidversion_code = property(get_androidversion_code)

    def get_androidversion_name(self):
        """
            Return the android version name
        """
        return self.androidversion["Name"]
    androidversion_name = property(get_androidversion_name)

    def get_files(self):
        """
            Return the files inside the APK
        """
        return self.zip.namelist()
    files = property(get_files)

    def get_files_types(self):
        """
            Return the files inside the APK with their types (by using python-magic)
        """
        try:
            import magic
        except ImportError:
            return {}

        l = {}

        builtin_magic = 0
        try:
            getattr(magic, "Magic")
        except AttributeError:
            builtin_magic = 1

        if builtin_magic:
            ms = magic.open(magic.MAGIC_NONE)
            ms.load()

            for i in self.get_files():
                l[ i ] = ms.buffer(self.zip.read(i))
        else:
            m = magic.Magic()
            for i in self.get_files():
                l[ i ] = m.from_buffer(self.zip.read(i))

        return l
    files_types = property(get_files_types)

    def get_raw(self):
        """
            Return raw bytes of the APK
        """
        return self.__raw
    raw = property(get_raw)

    def get_file(self, filename):
        """
            Return the raw data of the specified filename
        """
        try:
            return self.zip.read(filename)
        except KeyError:
            return ""

    def get_dex(self):
        """
            Return the raw data of the classes dex file
        """
        return self.get_file("classes.dex")
    dex = property(get_dex)

    def get_elements(self, tag_name, attribute):
        """
            Return elements in xml files which match with the tag name and the specific attribute

            @param tag_name: a string which specify the tag name
            @param attribute: a string which specify the attribute
        """
        l = []
        for i in self.xml:
            for item in self.xml[i].getElementsByTagName(tag_name):
                value = item.getAttribute(attribute)

                if len(value) > 0:
                    if value[0] == ".":
                        value = self.package + value
                    else:
                        v_dot = value.find(".")
                        if v_dot == 0:
                            value = self.package + "." + value
                        elif v_dot == -1:
                            value = self.package + "." + value

                l.append(str(value))
        return l

    def get_element(self, tag_name, attribute):
        """
            Return element in xml files which match with the tag name and the specific attribute

            @param tag_name: a string which specify the tag name
            @param attribute: a string which specify the attribute
        """
        l = []
        for i in self.xml:
            for item in self.xml[i].getElementsByTagName(tag_name):
                value = item.getAttribute(attribute)

                if len(value) > 0:
                    return value
        return None

    def get_activities(self):
        """
            Return the android:name attribute of all activities
        """
        return self.get_elements("activity", "android:name")
    activities = property(get_activities)

    def get_services(self):
        """
            Return the android:name attribute of all services
        """
        return self.get_elements("service", "android:name")
    services = property(get_services)

    def get_receivers(self):
        """
            Return the android:name attribute of all receivers
        """
        return self.get_elements("receiver", "android:name")
    receivers = property(get_receivers)

    def get_providers(self):
        """
            Return the android:name attribute of all providers
        """
        return self.get_elements("provider", "android:name")
    providers = property(get_providers)

    def get_permissions(self):
        """
            Return permissions
        """
        return self._permissions
    permissions = property(get_permissions)

    def get_min_sdk_version(self):
        """
            Return the android:minSdkVersion attribute
        """
        return self.get_element("uses-sdk", "android:minSdkVersion")
    min_sdk_version = property(get_min_sdk_version)

    def get_target_sdk_version(self):
        """
            Return the android:targetSdkVersion attribute
        """
        return self.get_element("uses-sdk", "android:targetSdkVersion")
    target_sdk_version = property(get_target_sdk_version)

    def get_libraries(self):
        """
            Return the android:name attributes for libraries
        """
        return self.get_elements("uses-library", "android:name")
    libraries = property(get_libraries)

    def show(self):
        print("FILES: ", self.get_files_types())

        print("ACTIVITIES: ", self.get_activities())
        print("SERVICES: ", self.get_services())
        print("RECEIVERS: ", self.get_receivers())
        print("PROVIDERS: ", self.get_providers())




import threading
from abc import ABC, abstractmethod

from airtestProject.abstractBase.OperateBase import OperateABC
from airtestProject.abstractBase.operateBaseImp.airtestOperate import myAirTest
from airtestProject.abstractBase.operateBaseImp.pocoOperate import myPoco
from airtestProject.poco.drivers.unity3d import UnityPoco
from airtestProject.factory.SingletonFactory import SingletonFactory


class AbstractFactory(ABC):
    def __init__(self):
        self._local = threading.local()

    @abstractmethod
    def create_operate_abc(self, **kwargs) -> OperateABC:
        pass


# 具体工厂
class PocoFactory(AbstractFactory, metaclass=SingletonFactory):
    def __init__(self):
        super().__init__()
        self._UnityPoco = None

    def create_operate_abc(self, **kwargs) -> OperateABC:
        if self._UnityPoco is None:
            self._UnityPoco = UnityPoco()
        if not hasattr(self._local, '_PageInstance'):
            self._local._PocoInstance = myPoco(self._UnityPoco)
        return self._local._PocoInstance


# 具体工厂
class AirTestFactory(AbstractFactory, metaclass=SingletonFactory):
    def __init__(self):
        super().__init__()

    def create_operate_abc(self, language=None, **kwargs) -> OperateABC:
        if not hasattr(self._local, '_AirTestInstance'):
            self._local._AirTestInstance = myAirTest(language)
        return self._local._AirTestInstance


def operate(factory_name: str, **kwargs) -> OperateABC:
    """

    :param factory_name: 工厂名称，可以是poco或air。
    :param kwargs: 可选参数。如果factory_name是air，可以传入language参数。设置语言需要在最开始设置，否者按照默认运行
    :return: 返回具体的工厂实例。
    """
    factories = {
        'poco': PocoFactory,
        'air': AirTestFactory
    }
    factory_class = factories[factory_name]
    factory = factory_class()
    product = factory.create_operate_abc(**kwargs)
    return product

# class operateFactoryOut:
#
#



# coding=utf-8

import time
from airtestProject.poco.drivers.unity3d.test.tutorial.case import TutorialCase


class Scroll2Tutorial(TutorialCase):
    def runTest(self):
        listView = self.poco('Scroll View')
        listView.focus([0.5, 0.8]).drag_to(listView.focus([0.5, 0.2]))
        time.sleep(1)


if __name__ == '__main__':
    import pocounit
    pocounit.main()



"""
自定义异常类
"""


class LRunException(Exception):
    pass


class NoSuchNodeException(LRunException):
    """
    This is NoSuchNodeException
    """
    pass


class InvalidOperationException(LRunException):
    """
    This is InvalidOperationException
    """
    pass


class TargetTimeout(LRunException):
    """
    This is TargetTimeout
    """
    pass


class ConnectError(LRunException):
    """
    This is ConnectError
    """
    pass


class AdbError(LRunException):
    """
    This is AdbError
    """
    pass


class InvalidParamError(LRunException):
    """
    This is InvalidParamError
    """
    pass


class FilteredRectsNotEmpty(Exception):
    def __init__(self, message="疑似资源缺失"):
        self.message = message
        super().__init__(self.message)

    def handle(self):
        print(self.message)



import json
import sys
if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

from ..exceptions import JSONRPCInvalidRequestException
from ..jsonrpc2 import (
    JSONRPC20Request,
    JSONRPC20BatchRequest,
    JSONRPC20Response,
    JSONRPC20BatchResponse,
)


class TestJSONRPC20Request(unittest.TestCase):

    """ Test JSONRPC20Request functionality."""

    def setUp(self):
        self.request_params = {
            "method": "add",
            "params": [1, 2],
            "_id": 1,
        }

    def test_correct_init(self):
        """ Test object is created."""
        JSONRPC20Request(**self.request_params)

    def test_validation_incorrect_no_parameters(self):
        with self.assertRaises(ValueError):
            JSONRPC20Request()

    def test_method_validation_str(self):
        self.request_params.update({"method": "add"})
        JSONRPC20Request(**self.request_params)

    def test_method_validation_not_str(self):
        self.request_params.update({"method": []})
        with self.assertRaises(ValueError):
            JSONRPC20Request(**self.request_params)

        self.request_params.update({"method": {}})
        with self.assertRaises(ValueError):
            JSONRPC20Request(**self.request_params)

    def test_method_validation_str_rpc_prefix(self):
        """ Test method SHOULD NOT starts with rpc. """
        self.request_params.update({"method": "rpc."})
        with self.assertRaises(ValueError):
            JSONRPC20Request(**self.request_params)

        self.request_params.update({"method": "rpc.test"})
        with self.assertRaises(ValueError):
            JSONRPC20Request(**self.request_params)

        self.request_params.update({"method": "rpccorrect"})
        JSONRPC20Request(**self.request_params)

        self.request_params.update({"method": "rpc"})
        JSONRPC20Request(**self.request_params)

    def test_params_validation_list(self):
        self.request_params.update({"params": []})
        JSONRPC20Request(**self.request_params)

        self.request_params.update({"params": [0]})
        JSONRPC20Request(**self.request_params)

    def test_params_validation_tuple(self):
        self.request_params.update({"params": ()})
        JSONRPC20Request(**self.request_params)

        self.request_params.update({"params": tuple([0])})
        JSONRPC20Request(**self.request_params)

    def test_params_validation_dict(self):
        self.request_params.update({"params": {}})
        JSONRPC20Request(**self.request_params)

        self.request_params.update({"params": {"a": 0}})
        JSONRPC20Request(**self.request_params)

    def test_params_validation_none(self):
        self.request_params.update({"params": None})
        JSONRPC20Request(**self.request_params)

    def test_params_validation_incorrect(self):
        self.request_params.update({"params": "str"})
        with self.assertRaises(ValueError):
            JSONRPC20Request(**self.request_params)

    def test_request_args(self):
        self.assertEqual(JSONRPC20Request("add").args, ())
        self.assertEqual(JSONRPC20Request("add", []).args, ())
        self.assertEqual(JSONRPC20Request("add", {"a": 1}).args, ())
        self.assertEqual(JSONRPC20Request("add", [1, 2]).args, (1, 2))

    def test_request_kwargs(self):
        self.assertEqual(JSONRPC20Request("add").kwargs, {})
        self.assertEqual(JSONRPC20Request("add", [1, 2]).kwargs, {})
        self.assertEqual(JSONRPC20Request("add", {}).kwargs, {})
        self.assertEqual(JSONRPC20Request("add", {"a": 1}).kwargs, {"a": 1})

    def test_id_validation_string(self):
        self.request_params.update({"_id": "id"})
        JSONRPC20Request(**self.request_params)

    def test_id_validation_int(self):
        self.request_params.update({"_id": 0})
        JSONRPC20Request(**self.request_params)

    def test_id_validation_null(self):
        self.request_params.update({"_id": "null"})
        JSONRPC20Request(**self.request_params)

    def test_id_validation_none(self):
        self.request_params.update({"_id": None})
        JSONRPC20Request(**self.request_params)

    def test_id_validation_float(self):
        self.request_params.update({"_id": 0.1})
        with self.assertRaises(ValueError):
            JSONRPC20Request(**self.request_params)

    def test_id_validation_incorrect(self):
        self.request_params.update({"_id": []})
        with self.assertRaises(ValueError):
            JSONRPC20Request(**self.request_params)

        self.request_params.update({"_id": ()})
        with self.assertRaises(ValueError):
            JSONRPC20Request(**self.request_params)

    def test_data_method_1(self):
        r = JSONRPC20Request("add")
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": None,
        })

    def test_data_method_2(self):
        r = JSONRPC20Request(method="add")
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": None,
        })

    def test_data_method_3(self):
        r = JSONRPC20Request("add", None)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": None,
        })

    def test_data_params_1(self):
        r = JSONRPC20Request("add", params=None, _id=None)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": None,
        })

    def test_data_params_2(self):
        r = JSONRPC20Request("add", [])
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "params": [],
            "id": None,
        })

    def test_data_params_3(self):
        r = JSONRPC20Request("add", ())
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "params": [],
            "id": None,
        })

    def test_data_params_4(self):
        r = JSONRPC20Request("add", (1, 2))
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "params": [1, 2],
            "id": None,
        })

    def test_data_params_5(self):
        r = JSONRPC20Request("add", {"a": 0})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "params": {"a": 0},
            "id": None,
        })

    def test_data_id_1(self):
        r = JSONRPC20Request("add", _id="null")
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": "null",
        })

    def test_data_id_1_notification(self):
        r = JSONRPC20Request("add", _id="null", is_notification=True)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
        })

    def test_data_id_2(self):
        r = JSONRPC20Request("add", _id=None)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": None,
        })

    def test_data_id_2_notification(self):
        r = JSONRPC20Request("add", _id=None, is_notification=True)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
        })

    def test_data_id_3(self):
        r = JSONRPC20Request("add", _id="id")
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": "id",
        })

    def test_data_id_3_notification(self):
        r = JSONRPC20Request("add", _id="id", is_notification=True)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
        })

    def test_data_id_4(self):
        r = JSONRPC20Request("add", _id=0)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": 0,
        })

    def test_data_id_4_notification(self):
        r = JSONRPC20Request("add", _id=0, is_notification=True)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
        })

    def test_is_notification(self):
        r = JSONRPC20Request("add")
        self.assertFalse(r.is_notification)

        r = JSONRPC20Request("add", _id=None)
        self.assertFalse(r.is_notification)

        r = JSONRPC20Request("add", _id="null")
        self.assertFalse(r.is_notification)

        r = JSONRPC20Request("add", _id=0)
        self.assertFalse(r.is_notification)

        r = JSONRPC20Request("add", is_notification=True)
        self.assertTrue(r.is_notification)

        r = JSONRPC20Request("add", is_notification=True, _id=None)
        self.assertTrue(r.is_notification)
        self.assertNotIn("id", r.data)

        r = JSONRPC20Request("add", is_notification=True, _id=0)
        self.assertTrue(r.is_notification)
        self.assertNotIn("id", r.data)

    def test_set_unset_notification_keep_id(self):
        r = JSONRPC20Request("add", is_notification=True, _id=0)
        self.assertTrue(r.is_notification)
        self.assertFalse("id" in r.data)

        r.is_notification = False
        self.assertFalse(r.is_notification)
        self.assertTrue("id" in r.data)
        self.assertEqual(r.data["id"], 0)

    def test_serialize_method_1(self):
        r = JSONRPC20Request("add")
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "id": None,
        }, json.loads(r.json))

    def test_serialize_method_2(self):
        r = JSONRPC20Request(method="add")
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "id": None,
        }, json.loads(r.json))

    def test_serialize_method_3(self):
        r = JSONRPC20Request("add", None)
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "id": None,
        }, json.loads(r.json))

    def test_serialize_params_1(self):
        r = JSONRPC20Request("add", params=None, _id=None)
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "id": None,
        }, json.loads(r.json))

    def test_serialize_params_2(self):
        r = JSONRPC20Request("add", [])
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "params": [],
            "id": None,
        }, json.loads(r.json))

    def test_serialize_params_3(self):
        r = JSONRPC20Request("add", ())
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "params": [],
            "id": None,
        }, json.loads(r.json))

    def test_serialize_params_4(self):
        r = JSONRPC20Request("add", (1, 2))
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "params": [1, 2],
            "id": None,
        }, json.loads(r.json))

    def test_serialize_params_5(self):
        r = JSONRPC20Request("add", {"a": 0})
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "params": {"a": 0},
            "id": None,
        }, json.loads(r.json))

    def test_serialize_id_1(self):
        r = JSONRPC20Request("add", _id="null")
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "id": "null",
        }, json.loads(r.json))

    def test_serialize_id_2(self):
        r = JSONRPC20Request("add", _id=None)
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "id": None,
        }, json.loads(r.json))

    def test_serialize_id_3(self):
        r = JSONRPC20Request("add", _id="id")
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "id": "id",
        }, json.loads(r.json))

    def test_serialize_id_4(self):
        r = JSONRPC20Request("add", _id=0)
        self.assertTrue({
            "jsonrpc": "2.0",
            "method": "add",
            "id": 0,
        }, json.loads(r.json))

    def test_from_json_request_no_id(self):
        str_json = json.dumps({
            "method": "add",
            "params": [1, 2],
            "jsonrpc": "2.0",
        })

        request = JSONRPC20Request.from_json(str_json)
        self.assertTrue(isinstance(request, JSONRPC20Request))
        self.assertEqual(request.method, "add")
        self.assertEqual(request.params, [1, 2])
        self.assertEqual(request._id, None)
        self.assertTrue(request.is_notification)

    def test_from_json_request_no_params(self):
        str_json = json.dumps({
            "method": "add",
            "jsonrpc": "2.0",
        })

        request = JSONRPC20Request.from_json(str_json)
        self.assertTrue(isinstance(request, JSONRPC20Request))
        self.assertEqual(request.method, "add")
        self.assertEqual(request.params, None)
        self.assertEqual(request._id, None)
        self.assertTrue(request.is_notification)

    def test_from_json_request_null_id(self):
        str_json = json.dumps({
            "method": "add",
            "jsonrpc": "2.0",
            "id": None,
        })

        request = JSONRPC20Request.from_json(str_json)
        self.assertTrue(isinstance(request, JSONRPC20Request))
        self.assertEqual(request.method, "add")
        self.assertEqual(request.params, None)
        self.assertEqual(request._id, None)
        self.assertFalse(request.is_notification)

    def test_from_json_request(self):
        str_json = json.dumps({
            "method": "add",
            "params": [0, 1],
            "jsonrpc": "2.0",
            "id": "id",
        })

        request = JSONRPC20Request.from_json(str_json)
        self.assertTrue(isinstance(request, JSONRPC20Request))
        self.assertEqual(request.method, "add")
        self.assertEqual(request.params, [0, 1])
        self.assertEqual(request._id, "id")
        self.assertFalse(request.is_notification)

    def test_from_json_invalid_request_jsonrpc(self):
        str_json = json.dumps({
            "method": "add",
        })

        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPC20Request.from_json(str_json)

    def test_from_json_invalid_request_method(self):
        str_json = json.dumps({
            "jsonrpc": "2.0",
        })

        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPC20Request.from_json(str_json)

    def test_from_json_invalid_request_extra_data(self):
        str_json = json.dumps({
            "jsonrpc": "2.0",
            "method": "add",
            "is_notification": True,
        })

        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPC20Request.from_json(str_json)

    def test_data_setter(self):
        request = JSONRPC20Request(**self.request_params)
        with self.assertRaises(ValueError):
            request.data = []

        with self.assertRaises(ValueError):
            request.data = ""

        with self.assertRaises(ValueError):
            request.data = None


class TestJSONRPC20BatchRequest(unittest.TestCase):

    """ Test JSONRPC20BatchRequest functionality."""

    def test_batch_request(self):
        request = JSONRPC20BatchRequest(
            JSONRPC20Request("devide", {"num": 1, "denom": 2}, _id=1),
            JSONRPC20Request("devide", {"num": 3, "denom": 2}, _id=2),
        )
        self.assertEqual(json.loads(request.json), [
            {"method": "devide", "params": {"num": 1, "denom": 2}, "id": 1,
             "jsonrpc": "2.0"},
            {"method": "devide", "params": {"num": 3, "denom": 2}, "id": 2,
             "jsonrpc": "2.0"},
        ])

    def test_from_json_batch(self):
        str_json = json.dumps([
            {"method": "add", "params": [1, 2], "jsonrpc": "2.0"},
            {"method": "mul", "params": [1, 2], "jsonrpc": "2.0"},
        ])

        requests = JSONRPC20BatchRequest.from_json(str_json)
        self.assertTrue(isinstance(requests, JSONRPC20BatchRequest))
        for r in requests:
            self.assertTrue(isinstance(r, JSONRPC20Request))
            self.assertTrue(r.method in ["add", "mul"])
            self.assertEqual(r.params, [1, 2])
            self.assertEqual(r._id, None)
            self.assertTrue(r.is_notification)

    def test_from_json_batch_one(self):
        str_json = json.dumps([
            {"method": "add", "params": [1, 2], "jsonrpc": "2.0", "id": None},
        ])

        requests = JSONRPC20Request.from_json(str_json)
        self.assertTrue(isinstance(requests, JSONRPC20BatchRequest))
        requests = list(requests)
        self.assertEqual(len(requests), 1)
        r = requests[0]
        self.assertTrue(isinstance(r, JSONRPC20Request))
        self.assertEqual(r.method, "add")
        self.assertEqual(r.params, [1, 2])
        self.assertEqual(r._id, None)
        self.assertFalse(r.is_notification)

    def test_response_iterator(self):
        requests = JSONRPC20BatchRequest(
            JSONRPC20Request("devide", {"num": 1, "denom": 2}, _id=1),
            JSONRPC20Request("devide", {"num": 3, "denom": 2}, _id=2),
        )
        for request in requests:
            self.assertTrue(isinstance(request, JSONRPC20Request))
            self.assertEqual(request.method, "devide")


class TestJSONRPC20Response(unittest.TestCase):

    """ Test JSONRPC20Response functionality."""

    def setUp(self):
        self.response_success_params = {
            "result": "",
            "_id": 1,
        }
        self.response_error_params = {
            "error": {
                "code": 1,
                "message": "error",
            },
            "_id": 1,
        }

    def test_correct_init(self):
        """ Test object is created."""
        JSONRPC20Response(**self.response_success_params)

    def test_validation_incorrect_no_parameters(self):
        with self.assertRaises(ValueError):
            JSONRPC20Response()

    def test_validation_incorrect_result_and_error(self):
        response = JSONRPC20Response(error={"code": 1, "message": ""})
        with self.assertRaises(ValueError):
            response.result = ""

    def test_validation_error_correct(self):
        JSONRPC20Response(**self.response_error_params)

    def test_validation_error_incorrect(self):
        self.response_error_params["error"].update({"code": "str"})
        with self.assertRaises(ValueError):
            JSONRPC20Response(**self.response_error_params)

    def test_validation_error_incorrect_no_code(self):
        del self.response_error_params["error"]["code"]
        with self.assertRaises(ValueError):
            JSONRPC20Response(**self.response_error_params)

    def test_validation_error_incorrect_no_message(self):
        del self.response_error_params["error"]["message"]
        with self.assertRaises(ValueError):
            JSONRPC20Response(**self.response_error_params)

    def test_validation_error_incorrect_message_not_str(self):
        self.response_error_params["error"].update({"message": 0})
        with self.assertRaises(ValueError):
            JSONRPC20Response(**self.response_error_params)

    def test_validation_id(self):
        response = JSONRPC20Response(**self.response_success_params)
        self.assertEqual(response._id, self.response_success_params["_id"])

    def test_validation_id_incorrect_type(self):
        response = JSONRPC20Response(**self.response_success_params)

        with self.assertRaises(ValueError):
            response._id = []

        with self.assertRaises(ValueError):
            response._id = {}

        with self.assertRaises(ValueError):
            response._id = 0.1

    def test_data_result(self):
        r = JSONRPC20Response(result="")
        self.assertEqual(json.loads(r.json), r.data)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "result": "",
            "id": None,
        })

    def test_data_result_id_none(self):
        r = JSONRPC20Response(result="", _id=None)
        self.assertEqual(json.loads(r.json), r.data)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "result": "",
            "id": None,
        })

    def test_data_result_id(self):
        r = JSONRPC20Response(result="", _id=0)
        self.assertEqual(json.loads(r.json), r.data)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "result": "",
            "id": 0,
        })

    def test_data_error(self):
        r = JSONRPC20Response(error={"code": 0, "message": ""})
        self.assertEqual(json.loads(r.json), r.data)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "error": {
                "code": 0,
                "message": "",
            },
            "id": None,
        })

    def test_data_error_id_none(self):
        r = JSONRPC20Response(error={"code": 0, "message": ""}, _id=None)
        self.assertEqual(json.loads(r.json), r.data)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "error": {
                "code": 0,
                "message": "",
            },
            "id": None,
        })

    def test_data_error_id(self):
        r = JSONRPC20Response(error={"code": 0, "message": ""}, _id=0)
        self.assertEqual(json.loads(r.json), r.data)
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "error": {
                "code": 0,
                "message": "",
            },
            "id": 0,
        })

    def test_data_setter(self):
        response = JSONRPC20Response(**self.response_success_params)
        with self.assertRaises(ValueError):
            response.data = []

        with self.assertRaises(ValueError):
            response.data = ""

        with self.assertRaises(ValueError):
            response.data = None


class TestJSONRPC20BatchResponse(unittest.TestCase):

    """ Test JSONRPC20BatchResponse functionality."""

    def test_batch_response(self):
        response = JSONRPC20BatchResponse(
            JSONRPC20Response(result="result", _id=1),
            JSONRPC20Response(error={"code": 0, "message": ""}, _id=2),
        )
        self.assertEqual(json.loads(response.json), [
            {"result": "result", "id": 1, "jsonrpc": "2.0"},
            {"error": {"code": 0, "message": ""}, "id": 2, "jsonrpc": "2.0"},
        ])

    def test_response_iterator(self):
        responses = JSONRPC20BatchResponse(
            JSONRPC20Response(result="result", _id=1),
            JSONRPC20Response(result="result", _id=2),
        )
        for response in responses:
            self.assertTrue(isinstance(response, JSONRPC20Response))
            self.assertEqual(response.result, "result")

    def test_batch_response_data(self):
        response = JSONRPC20BatchResponse(
            JSONRPC20Response(result="result", _id=1),
            JSONRPC20Response(result="result", _id=2),
            JSONRPC20Response(result="result"),
        )
        self.assertEqual(response.data, [
            {"id": 1, "jsonrpc": "2.0", "result": "result"},
            {"id": 2, "jsonrpc": "2.0", "result": "result"},
            {"id": None, "jsonrpc": "2.0", "result": "result"},
        ])



# -*- coding: utf-8 -*-
import traceback
from airtestProject.airtest import aircv


class BaseCap(object):
    """
    Base class for all screenshot methods
    所有屏幕截图方法的基类
    """

    def __init__(self, adb, *args, **kwargs):
        self.adb = adb

    def get_frame_from_stream(self):
        """
        Get a frame of the current screen from the mobile screen stream

        从手机画面流中，获取一张当前屏幕截图

        Returns: frame_data

        """
        raise NotImplementedError

    def get_frame(self):
        # 获得单张屏幕截图
        return self.get_frame_from_stream()

    def teardown_stream(self):
        pass

    def snapshot(self, ensure_orientation=True, *args, **kwargs):
        """
        Take a screenshot and convert it into a cv2 image object

        获取一张屏幕截图，并转化成cv2的图像对象

        Returns: numpy.ndarray

        """
        screen = self.get_frame_from_stream()
        try:
            screen = aircv.utils.string_2_img(screen)
        except Exception:
            # may be black/locked screen or other reason, print exc for debugging
            traceback.print_exc()
            return None
        return screen



"""
根据Excel表格生成python测试用例
"""
import os
import openpyxl

from airtestProject import config
from airtestProject.commons.utils.excelPicture import get_id_name
from utils import *


# Utils = Utils()


def get_execls(project):
    """
    获取Excel文件
    return test_case_path_list
    """
    test_case_path = os.path.join(config.ROOT_PATH, "cases", project)
    test_case_path_list = []
    for root, pycache, files in os.walk(test_case_path):
        for file in files:
            if file.endswith('.xlsx') or file.endswith('.xls'):
                test_case_path_name = root + '\\' + file
                test_case_path_list.append(test_case_path_name)
    print(test_case_path_list)
    return test_case_path_list


def parse_excel(project="zg", page=0):
    pathList = get_execls(project)
    test_case_data = {}
    test_case_dic = {}
    if pathList is None:
        return
    for file in pathList:
        file_name = file.split('\\')[-1].split('.')[0]
        wb = openpyxl.load_workbook(file)
        sheet = wb.worksheets[page]
        result_dict = {}
        for row in range(2, sheet.max_row + 1):
            key = sheet.cell(row=row, column=1).value
            if key in result_dict:
                # 如果已经存在，将当前行号添加到对应的list中
                result_dict[key].append(row)
            else:
                if key is None:
                    continue
                # 如果不存在，创建一个新的list，并将行号添加进去
                result_dict[key] = [row]
        for key, vals in result_dict.items():
            operate_dic = {}
            for val in vals:
                operate_key = sheet.cell(val, 2).value
                operate_value = sheet.cell(val, 3).value
                if "_xlfn.DISPIMG" in str(operate_value):
                    operate_dic[operate_key] = [operate_value[1::]]
                elif ',' in str(operate_value):
                    value = operate_value.split(',')
                    operate_dic[operate_key] = value
                else:
                    operate_dic[operate_key] = [operate_value]
            test_case_data[key] = operate_dic
        test_case_dic[file_name] = test_case_data

        ## 把图片id替换为图片路径
        id_name = get_id_name(project, file, page)
        for key, value in test_case_dic[file_name].items():
            for k, v in value.items():
                if v[0] in id_name.keys():
                    test_case_dic[file_name][key][k][0] = id_name[v[0]]


    print(test_case_dic)
    return test_case_dic


if __name__ == '__main__':
    parse_excel("zg", 0)



# coding=utf-8

import re
from .exceptions import NoSuchComparatorException

__author__ = 'lxn3032'
__all__ = ['IMatcher', 'DefaultMatcher', 'EqualizationComparator', 'RegexpComparator']


class IMatcher(object):
    def match(self, cond, node):
        """
        Test whether or not the node matches the given condition.

        Args:
            cond (:obj:`tuple`): query expression
            node (:py:class:`inherit from AbstractNode <poco.sdk.AbstractNode>`): node to be tested

        Returns:
            bool: True if matches otherwise False.
        """

        raise NotImplementedError


class EqualizationComparator(object):
    """
    Compare two objects using the native equivalence (==) comparison operator
    """

    def compare(self, l, r):
        return l == r


class RegexpComparator(object):
    """
    Compare two objects using regular expression. Available only when the original value is string type. It always
    returns False if the original value or given pattern are not :obj:`str` type.
    """

    def compare(self, origin, pattern):
        """
        Args:
            origin (:obj:`str`): original string
            pattern (:obj:`str`): Regexp pattern string

        Returns:
            bool: True if matches otherwise False.
        """

        if origin is None or pattern is None:
            return False
        return re.match(pattern, origin) is not None


class DefaultMatcher(IMatcher):
    """
    Default matcher implementation for poco hierarchy traversing. Including logical query condition and predicate
    expression. When traversing through the hierarchy tree, matcher will apply the match method on each node of the tree.

    The formal definition of query condition as follows::

        expr := (op0, (expr0, expr1, ...))  
        expr := (op1, (arg1, arg2))  

    - ``op0``:obj:`str` is logical operator ('or' or 'and') which has the same semantics as in python, e.g. 'or'
      means this expression/condition matches if any of the exprN matches
    - ``op1``:obj:`str` is comparator, can be one of as follows::

        op1 := 'attr='
        op1 := 'attr.*='
        op1 := ... (other customized)

      - ``attr=`` corresponds to :py:class:`EqualizationComparator <poco.sdk.DefaultMatcher.EqualizationComparator>`.
      - ``attr.*=`` corresponds to :py:class:`RegexpComparator <poco.sdk.DefaultMatcher.RegexpComparator>`.
      
      The ``op1`` must be a string. The ``Matcher`` will help to map to ``Comparator`` object.
    """

    def __init__(self):
        super(DefaultMatcher, self).__init__()
        self.comparators = {
            'attr=': EqualizationComparator(),
            'attr.*=': RegexpComparator(),
        }

    def match(self, cond, node):
        """
        See Also: :py:meth:`IMatcher.match <poco.sdk.DefaultMatcher.IMatcher.match>`
        """

        op, args = cond

        # 条件匹配
        if op == 'and':
            for arg in args:
                if not self.match(arg, node):
                    return False
            return True

        if op == 'or':
            for arg in args:
                if self.match(arg, node):
                    return True
            return False

        # 属性匹配
        comparator = self.comparators.get(op)
        if comparator:
            attribute, value = args
            targetValue = node.getAttr(attribute)
            return comparator.compare(targetValue, value)

        raise NoSuchComparatorException(op, 'poco.sdk.DefaultMatcher')



# -*- coding: utf-8 -*-

"""
error classes
"""

class BaseError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class AirtestError(BaseError):
    """
        This is Airtest BaseError
    """
    pass


class InvalidMatchingMethodError(BaseError):
    """
        This is InvalidMatchingMethodError BaseError
        When an invalid matching method is used in settings.
    """
    pass


class TargetNotFoundError(AirtestError):
    """
        This is TargetNotFoundError BaseError
        When something is not found
    """
    pass


class ScriptParamError(AirtestError):
    """
        This is ScriptParamError BaseError
        When something goes wrong
    """
    pass


class AdbError(Exception):
    """
        This is AdbError BaseError
        When ADB have something wrong
    """
    def __init__(self, stdout, stderr):
        self.stdout = stdout
        self.stderr = stderr

    def __str__(self):
        return "stdout[%s] stderr[%s]" % (self.stdout, self.stderr)


class AdbShellError(AdbError):
    """
        adb shell error
    """
    pass


class DeviceConnectionError(BaseError):
    """
        device connection error
    """
    DEVICE_CONNECTION_ERROR = r"error:\s*((device \'\S+\' not found)|(cannot connect to daemon at [\w\:\s\.]+ Connection timed out))"
    pass

class NoDeviceError(BaseError):
    """
        When no device is connected
    """
    pass


class ICmdError(Exception):
    """
        This is ICmdError BaseError
        When ICmd have something wrong
    """    
    def __init__(self, stdout, stderr):
        self.stdout = stdout
        self.stderr = stderr

    def __str__(self):
        return "stdout[%s] stderr[%s]" %(self.stdout, self.stderr)


class ScreenError(BaseError):
    """
    When the screen capture method(Minicap/Javacap/ScreenProxy) has something wrong
    """
    pass


class MinicapError(ScreenError):
    """
        This is MinicapError BaseError
        When Minicap have something wrong
    """    
    pass


class MinitouchError(BaseError):
    """
        This is MinitouchError BaseError
        When Minicap have something wrong
    """    
    pass


class PerformanceError(BaseError):
    pass


class LocalDeviceError(BaseError):
    """
    Custom exception for calling a method on a non-local iOS device.
    """
    def __init__(self, value="Can only use this method on a local device."):
        super().__init__(value)



# coding=utf-8

import time
from airtestProject.poco.drivers.unity3d.test.tutorial.case import TutorialCase


class ButtonsAndLabelsTutorial(TutorialCase):
    def runTest(self):
        self.poco('btn_start').click()
        self.poco(text='basic').click()

        star = self.poco('star_single')
        if star.exists():
            pos = star.get_position()
            input_field = self.poco('pos_input')
            time.sleep(1)
            input_field.set_text('x={:.02f}, y={:.02f}'.format(*pos))
            time.sleep(3)

        title = self.poco('title').get_text()
        if title == 'Basic test':
            self.poco('btn_back', type='Button').click()
            self.poco('btn_back', type='Button').click()


if __name__ == '__main__':
    import pocounit
    pocounit.main()



# _*_ coding: UTF-8 _*_
from airtestProject.airtest.core.api import *
from airtestProject.airtest.cli.parser import cli_setup


# from PIL import Image
# import matplotlib.pyplot as plt
import sys
from airtestProject.airtest.core.android.adb import ADB

# auto_setup(__file__,logdir=False,devices= ["Android://127.0.0.1:5037/46HDU19307002391"])


class startDeploy():
    """
    定义启动类
    """

    def __init__(self):
        """
        定义部署元素
        """
        #airtest输入法
        self.ym = "com.netease.nie.yosemite/.ime.ImeService"
        self.ime_list = []

        #相对坐标
        self.w = ()
        self.h = ()

        #安装包相关
        self.pwd = os.path.dirname(__file__)
        # self.pkg = "com.sanqi.odin.uwa.mono"

        #连接设备号
        # self.device_list = [
        #     ["Android://127.0.0.1:5037/46HDU19307002391"],#荣耀V20
        #     ["Android://127.0.0.1:5037/R5CTB17E1EX"]
        # ]
        self.url = "Android://127.0.0.1:5037/"

    def get_deviceId(self):
        """
        获取device
        """
        device = os.popen("adb devices").readlines()
        device_id = device[1]

        return self.url + device_id.split()[0]

    def get_auto_setup(self,device_list):
        """
        环境接口
        """
        return auto_setup(__file__, logdir=False, devices=[device_list])

    def get_url(self):
        return self.url

    def get_current_resolution(self):
        """
        相对坐标
        """
        self.w , self.h = device().get_current_resolution()
        return self.w , self.h

    # def switch_ime(self, param):
    #     """
    #     切换输入法
    #     """
    #     list = device().yosemite_ime._get_ime_list()
    #     def set_ime(ime):
    #         shell("ime enable " + ime)
    #         shell("ime set " + ime)
    #
    #     self.ime_list.append(self.ym)
    #
    #     for ime in list:
    #         if ime != self.ym:
    #             self.ime_list.append(ime)
    #             break
    #
    #     # 切换输入法
    #     if param == const.IME_YOSEMITE :
    #         set_ime(self.ime_list[0]) #Yosemite
    #
    #     else:
    #         set_ime(self.ime_list[1]) #手机自带


# dep = startDeploy()
# # 获取连接设备
# deviceId = dep.get_deviceId()
# print(deviceId)
# # startDeploy().get_auto_setup("Android://127.0.0.1:5037/46HDU19307002391")
# startDeploy().get_auto_setup("Android://127.0.0.1:5037/R5CTB17E1EX")
# st = startDeploy()
# # startDeploy().get_ime("YOSEMITE")
# 切换输入法
# str = startDeploy().get_deviceId()
# print(get_deviceId)
# startDeploy().get_auto_setup(str)
# log(str+'已连上')



#!-*- coding = utf-8 -*-
# @Time : 2024/4/7 2:44
# @Author : 苏嘉浩
# @File : OperateBase.py
# @Software : PyCharm
import os
import traceback
import six

from airtestProject.airtest.core.helper import G
from airtestProject.airtest.core.settings import Settings as ST


def log_error_plus(arg, timestamp=None, desc="", snapshot=False):
    """
    Examples:
        >>> log("hello world", snapshot=True)
        >>> log({"key": "value"}, timestamp=time.time(), desc="log dict")

    """
    from airtestProject.airtest.core.cv import try_log_screen
    if G.LOGGER:
        depth = 0
        if snapshot:
            # 如果指定了snapshot参数，强制保存一张图片
            save_image = ST.SAVE_IMAGE
            ST.SAVE_IMAGE = True
            try:
                try_log_screen(depth=2)
            except AttributeError:
                # if G.DEVICE is None
                pass
            else:
                depth = 1
            finally:
                ST.SAVE_IMAGE = save_image
            G.LOGGER.log("info", {
                "name": desc or arg.__class__.__name__,
                "traceback": arg,
            }, depth=depth, timestamp=timestamp)


def get_folder_path_up(current_path, folder_name):
    """
    向上寻找文件夹路径，找到后返回对应路径。
    :param current_path: 当前文件路径
    :param folder_name: 需要寻找的文件夹名字
    :return: 返回找到的文件夹
    """
    result_path = os.path.dirname(os.path.abspath(current_path))
    while True:
        if os.path.exists(os.path.join(result_path, folder_name)):
            # 当前文件路径下有对应文件夹路径
            result_path = os.path.join(result_path, folder_name)
            break
        elif result_path == os.path.dirname(result_path):
            # 去到根目录还未找到文件夹路径
            raise Exception(f"未找到 '{folder_name}' 文件夹，需要在根目录下创建{folder_name}文件夹")
        else:
            # 向上查找文件夹路径
            result_path = os.path.dirname(result_path)
    return result_path


def get_folder_path_down(current_path, folder_name):
    # 检查当前路径下是否有 'img' 文件夹
    if folder_name in os.listdir(current_path):
        return os.path.join(current_path, folder_name)

    # 在每个子目录中递归查找
    for subdir in os.listdir(current_path):
        full_subdir = os.path.join(current_path, subdir)
        if os.path.isdir(full_subdir):
            result = get_folder_path_down(current_path, folder_name)
            if result is not None:
                return result

    # 如果在当前路径及其所有子目录中都没有找到 'img' 文件夹，返回 None
    return None


def get_screenshot_cv():
    from airtestProject.airtest.core.helper import G
    return G.DEVICE.snapshot(quality=ST.SNAPSHOT_QUALITY)



# -*- coding: utf-8 -*-
import threading
import traceback
import time
import wda
from airtestProject.airtest.core.ios.constant import ROTATION_MODE
from airtestProject.airtest.utils.snippet import reg_cleanup, on_method_ready
from airtestProject.airtest.utils.logger import get_logger


LOGGING = get_logger(__name__)


class RotationWatcher(object):
    """
    RotationWatcher class
    """

    def __init__(self, iosHandle):
        self.iosHandle = iosHandle
        self.ow_callback = []
        self.roundProcess = None
        self._stopEvent = threading.Event()
        self.last_result = None
        reg_cleanup(self.teardown)

    @on_method_ready('start')
    def get_ready(self):
        pass

    def _install_and_setup(self):
        """
        Install and setup the RotationWatcher package

        Raises:
            RuntimeError: if any error occurs while installing the package

        Returns:
            None

        """
        # fetch orientation result
        self.last_result = None
        # reg_cleanup(self.ow_proc.kill)

    def teardown(self):
        # if has roataion watcher stop it
        if self.roundProcess:
            self._stopEvent.set()
        self.ow_callback = []

    def start(self):
        """
        Start the RotationWatcher daemon thread

        Returns:
            None

        """
        self._install_and_setup()

        def _refresh_by_ow():
            try:
                return self.get_rotation()
            except Exception:
                # 重试5次，如果还是失败就返回None，终止线程
                for i in range(5):
                    try:
                        self.iosHandle._fetch_new_session()
                        return self.get_rotation()
                    except:
                        time.sleep(2)
                        continue
                LOGGING.info("orientationWatcher has ended")
                return None

        def _run():
            while not self._stopEvent.isSet():
                time.sleep(1)
                ori = _refresh_by_ow()
                if ori is None:
                    break
                elif self.last_result == ori:
                    continue
                LOGGING.info('update orientation %s->%s' %
                             (self.last_result, ori))
                self.last_result = ori

                # exec cb functions
                for cb in self.ow_callback:
                    try:
                        cb(ori)
                    except:
                        LOGGING.error("cb: %s error" % cb)
                        traceback.print_exc()

        self.roundProcess = threading.Thread(
            target=_run, name="rotationwatcher")
        # self._t.daemon = True
        self.roundProcess.start()

    def reg_callback(self, ow_callback):
        """

        Args:
            ow_callback:

        Returns:

        """
        """方向变化的时候的回调函数，参数一定是ori，如果断掉了，ori传None"""
        self.ow_callback.append(ow_callback)

    def get_rotation(self):
        return self.iosHandle.get_orientation()


class XYTransformer(object):
    """
    transform the coordinates (x, y) by orientation (upright <--> original)
    """
    @staticmethod
    def up_2_ori(tuple_xy, tuple_wh, orientation):
        """
        Transform the coordinates upright --> original

        Args:
            tuple_xy: coordinates (x, y)
            tuple_wh: current screen width and height
            orientation: orientation

        Returns:
            transformed coordinates (x, y)

        """
        x, y = tuple_xy
        w, h = tuple_wh

        # no need to do changing
        # ios touch point same way of image

        if orientation == wda.LANDSCAPE:
            x, y = h-y, x
        elif orientation == wda.LANDSCAPE_RIGHT:
            x, y = y, w-x
        elif orientation == wda.PORTRAIT_UPSIDEDOWN:
            x, y = w-x, h-y
        elif orientation == wda.PORTRAIT:
            x, y = x, y
        return x, y

    @staticmethod
    def ori_2_up(tuple_xy, tuple_wh, orientation):
        """
        Transform the coordinates original --> upright

        Args:
            tuple_xy: coordinates (x, y)
            tuple_wh: current screen width and height
            orientation: orientation

        Returns:
            transformed coordinates (x, y)

        """
        x, y = tuple_xy
        w, h = tuple_wh

        # Only in the ipad+home interface,
        # the vertical screen coordinates need to be converted to display coordinates
        if orientation == wda.LANDSCAPE:
            x, y = y, h - x
        elif orientation == wda.LANDSCAPE_RIGHT:
            x, y = w - y, x
        elif orientation == wda.PORTRAIT_UPSIDEDOWN:
            x, y = w - x, h - y

        return x, y



# _*_ coding:UTF-8 _*_
import time
from threading import Thread, Event
from six.moves import queue
from .logger import get_logger
LOGGING = get_logger(__name__)


class NonBlockingStreamReader:

    def __init__(self, stream, raise_EOF=False, print_output=True, print_new_line=True, name=None, auto_kill=False):
        '''
        stream: the stream to read from.
                Usually a process' stdout or stderr.
        raise_EOF: if True, raise an UnexpectedEndOfStream
                when stream is EOF before kill
        print_output: if True, print when readline
        '''
        self._s = stream
        self._q = queue.Queue()
        self._lastline = None
        self.name = name or id(self)

        def _populateQueue(stream, queue, kill_event):
            '''
            Collect lines from 'stream' and put them in 'queue'.
            '''
            while not kill_event.is_set():
                line = stream.readline()
                if line is not None:
                    queue.put(line)
                    if print_output:
                        # print only new line
                        if print_new_line and line == self._lastline:
                            continue
                        self._lastline = line
                        LOGGING.debug("[%s]%s" % (self.name, repr(line.strip())))
                    if auto_kill and line == b"":
                        self.kill()
                elif kill_event.is_set():
                    break
                elif raise_EOF:
                    raise UnexpectedEndOfStream
                else:
                    break

        self._kill_event = Event()
        self._t = Thread(target=_populateQueue, args=(self._s, self._q, self._kill_event), name="nbsp_%s"%self.name)
        self._t.daemon = True
        self._t.start()  # start collecting lines from the stream

    def readline(self, timeout=None):
        try:
            return self._q.get(block=timeout is not None, timeout=timeout)
        except queue.Empty:
            return None

    def read(self, timeout=0):
        time.sleep(timeout)
        lines = []
        while True:
            line = self.readline()
            if line is None:
                break
            lines.append(line)
        return b"".join(lines)

    def kill(self):
        self._kill_event.set()


class UnexpectedEndOfStream(Exception):
    pass



# coding=utf-8
import re
from airtestProject.airtest.core.android.yosemite import Yosemite
from .constant import YOSEMITE_PACKAGE
from airtestProject.airtest.core.error import AdbError
from airtestProject.airtest.utils.snippet import escape_special_char
from .constant import YOSEMITE_IME_SERVICE
from six import text_type
import requests
import time


def ensure_unicode(value):
    """
    Decode UTF-8 values

    Args:
        value: value to be decoded

    Returns:
        decoded valued

    """
    if type(value) is not text_type:
        try:
            value = value.decode('utf-8')
        except UnicodeDecodeError:
            value = value.decode('gbk')
    return value


class CustomIme(object):
    """
    Input Methods Class Object
    """

    def __init__(self, adb, apk_path, service_name):
        super(CustomIme, self).__init__()
        self.adb = adb
        self.apk_path = apk_path
        self.service_name = service_name
        self.started = False

    def _get_ime_list(self):
        """
        Return all the input methods on the device

        Returns:
            list of all input methods on the device

        """
        out = self.adb.shell("ime list -a")
        m = re.findall("mId=(.*?/.*?) ", out)
        return m

    def __enter__(self):
        self.start()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.end()

    def start(self):
        """
        Enable input method

        Returns:
            None

        """
        try:
            self.default_ime = self.adb.shell("settings get secure default_input_method").strip()
        except AdbError:
            # settings cmd not found for older phones, e.g. Xiaomi 2A
            # /system/bin/sh: settings: not found
            self.default_ime = None
        self.ime_list = self._get_ime_list()
        if self.service_name not in self.ime_list:
            if self.apk_path:
                self.device.install_app(self.apk_path)
        if self.default_ime != self.service_name:
            self.adb.shell("ime enable %s" % self.service_name)
            self.adb.shell("ime set %s" % self.service_name)
        self.started = True

    def end(self):
        """
        Disable input method

        Returns:
            None

        """
        if self.default_ime and self.default_ime != self.service_name:
            self.adb.shell("ime disable %s" % self.service_name)
            self.adb.shell("ime set %s" % self.default_ime)
        self.started = False

    def text(self, value):
        raise NotImplementedError


class YosemiteIme(CustomIme):
    """
    Yosemite Input Method Class Object
    """

    def __init__(self, adb):
        super(YosemiteIme, self).__init__(adb, None, YOSEMITE_IME_SERVICE)
        self.yosemite = Yosemite(adb)

    def start(self):
        self.yosemite.get_ready()
        super(YosemiteIme, self).start()

    def text(self, value):
        """
        Input text with Yosemite input method

        Args:
            value: text to be inputted

        Returns:
            output form `adb shell` command

        """
        if not self.started:
            self.start()
            installed_version = self.adb.get_package_version(YOSEMITE_PACKAGE)
            if installed_version is not None and int(installed_version) >= int(430):
                local_port = self.adb.get_available_forward_local()
                self.adb.forward(local=f'tcp:{local_port}', remote="tcp:8181", no_rebind=False)
                for i in range(15):
                    try:
                        url = f'http://{self.adb.host}:{local_port}/ime_onStartInput'
                        ret = requests.get(url, timeout=3)
                    except Exception as e:
                        time.sleep(1)
                        if i % 3 == 0:  # 超时未启动，再尝试启动
                            print(f"Trying to initialize input method again.")
                            self.start()
                    else:
                        if 'ime_onStartInput=ok' in ret.text:
                            self.adb.remove_forward(local=f'tcp:{local_port}')
                            break
                else:
                    self.adb.remove_forward(local=f'tcp:{local_port}')
                    raise AdbError("IME Error", "IME initialization timed out.")

        # 更多的输入用法请见 https://github.com/macacajs/android-unicode#use-in-adb-shell
        value = ensure_unicode(value)
        value = escape_special_char(value)
        self.adb.shell(f"am broadcast -a ADB_INPUT_TEXT --es msg {value}")

    def code(self, code):
        """
        Sending editor action

        Args:
            code: editor action code, e.g., 2 = IME_ACTION_GO, 3 = IME_ACTION_SEARCH
                Editor Action Code Ref: http://developer.android.com/reference/android/view/inputmethod/EditorInfo.html

        Returns:
            output form `adb shell` command

        """
        if not self.started:
            self.start()
        self.adb.shell("am broadcast -a ADB_EDITOR_CODE --ei code {}".format(str(code)))



# coding=utf-8

import time
from airtestProject.poco.drivers.unity3d.test.tutorial.case import TutorialCase


class WaitAnyUITutorial(TutorialCase):
    def runTest(self):
        self.poco(text='wait UI 2').click()

        blue_fish = self.poco('fish_area').child('blue')
        yellow_fish = self.poco('fish_area').child('yellow')
        shark = self.poco('fish_area').child('black')

        self.poco.wait_for_all([blue_fish, yellow_fish, shark])
        self.poco('btn_back').click()
        time.sleep(2.5)


if __name__ == '__main__':
    import pocounit
    pocounit.main()



# -*- coding: utf-8 -*-

import time
import socket
import subprocess
import numpy
import mss
import psutil
from functools import wraps
import win32api
import win32clipboard
import pywintypes  # noqa
import os

from pywinauto.application import Application
from pywinauto import mouse, keyboard
from pywinauto.win32structures import RECT

from airtestProject.airtest.core.win.ctypesinput import key_press, key_release

from airtestProject.airtest import aircv
from airtestProject.airtest.aircv.screen_recorder import ScreenRecorder, resize_by_max, get_max_size
from airtestProject.airtest.core.device import Device
from airtestProject.airtest.core.settings import Settings as ST
from airtestProject.airtest.utils.snippet import get_absolute_coordinate
from airtestProject.airtest.utils.logger import get_logger

LOGGING = get_logger(__name__)

def require_app(func):
    @wraps(func)
    def wrapper(inst, *args, **kwargs):
        if not inst.app:
            raise RuntimeError("Connect to an application first to use %s" % func.__name__)
        return func(inst, *args, **kwargs)
    return wrapper


class Windows(Device):
    """Windows client."""

    def __init__(self, handle=None, dpifactor=1, **kwargs):
        super(Windows, self).__init__()
        self.app = None
        self.handle = int(handle) if handle else None
        # windows high dpi scale factor, no exact way to auto detect this value for a window
        # reference: https://msdn.microsoft.com/en-us/library/windows/desktop/mt843498(v=vs.85).aspx
        self._dpifactor = float(dpifactor)
        self._app = Application()
        self._top_window = None
        self._focus_rect = (0, 0, 0, 0)
        self.mouse = mouse
        self.keyboard = keyboard
        self._init_connect(handle, kwargs)

        self.screen = mss.mss()
        self.monitor = self.screen.monitors[0]  # 双屏的时候，self.monitor为整个双屏
        self.main_monitor = self.screen.monitors[1]  # 双屏的时候，self.main_monitor为主屏

    @property
    def uuid(self):
        return self.handle

    def _init_connect(self, handle, kwargs):
        if handle:
            self.connect(handle=handle, **kwargs)
        elif kwargs:
            self.connect(**kwargs)

    def connect(self, handle=None, **kwargs):
        """
        Connect to window and set it foreground

        Args:
            **kwargs: optional arguments

        Returns:
            None

        """
        if handle:
            handle = int(handle)
            self.app = self._app.connect(handle=handle)
            self._top_window = self.app.window(handle=handle).wrapper_object()
        else:
            for k in ["process", "timeout"]:
                if k in kwargs:
                    kwargs[k] = int(kwargs[k])
            self.app = self._app.connect(**kwargs)
            self._top_window = self.app.top_window().wrapper_object()
        if kwargs.get("foreground", True) in (True, "True", "true"):
            self.set_foreground()

    def shell(self, cmd):
        """
        Run shell command in subprocess

        Args:
            cmd: command to be run

        Raises:
            subprocess.CalledProcessError: when command returns non-zero exit status

        Returns:
            command output as a byte string

        """
        return subprocess.check_output(cmd, shell=True)

    def snapshot(self, filename=None, quality=10, max_size=None):
        """
        Take a screenshot and save it in ST.LOG_DIR folder

        Args:
            filename: name of the file to give to the screenshot, {time}.jpg by default
            quality: The image quality, integer in range [1, 99]
            max_size: the maximum size of the picture, e.g 1200

        Returns:
            display the screenshot

        """
        if self.app:
            rect = self.get_rect()
            rect = self._fix_image_rect(rect)
            monitor = {"top": rect.top, "left": rect.left, "width": rect.right - rect.left - abs(self.monitor["left"]),
                       "height": rect.bottom - rect.top, "monitor": 1}
        else:
            monitor = self.screen.monitors[0]
        with mss.mss() as sct:
            sct_img = sct.grab(monitor)
            screen = numpy.array(sct_img, dtype=numpy.uint8)[...,:3]
            if filename:
                aircv.imwrite(filename, screen, quality, max_size=max_size)
            return screen

    def _fix_image_rect(self, rect):
        """Fix rect in image."""
        # 将rect 转换为左上角为(0,0), 与图片坐标对齐，另外left不用重新计算
        rect.right = rect.right - self.monitor["left"]
        rect.top = rect.top - self.monitor["top"]
        rect.bottom = rect.bottom - self.monitor["top"]
        return rect

    def keyevent(self, keyname, **kwargs):
        """
        Perform a key event

        References:
            https://pywinauto.readthedocs.io/en/latest/code/pywinauto.keyboard.html

        Args:
            keyname: key event
            **kwargs: optional arguments

        Returns:
            None

        """
        self.keyboard.SendKeys(keyname)

    def text(self, text, **kwargs):
        """
        Input text

        Args:
            text: text to input
            **kwargs: optional arguments

        Returns:
            None

        """
        self.keyevent(text)

    def _fix_op_pos(self, pos):
        """Fix operation position."""
        # 如果是全屏的话，就进行双屏修正，否则就正常即可
        if not self.handle:
            pos = list(pos)
            pos[0] = pos[0] + self.monitor["left"]
            pos[1] = pos[1] + self.monitor["top"]

        return pos

    def key_press(self, key):
        """Simulates a key press event.

        Sends a scancode to the computer to report which key has been pressed.
        Some games use DirectInput devices, and respond only to scancodes, not
        virtual key codes. You can simulate DirectInput key presses using this
        method, instead of the keyevent() method, which uses virtual key
        codes.

        :param key: A string indicating which key to be pressed.
                    Available key options are:
                    {'ESCAPE', '1', '2', '3', '4', '5', '6', '7', '8', '9',
                    '0', '-', '=', 'BACKSPACE', 'TAB', 'Q', 'W', 'E', 'R', 'T',
                    'Y', 'U', 'I', 'O', 'P', '[', ']', 'ENTER', 'LCTRL', 'A',
                    'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ';', "'", '`',
                    'LSHIFT', 'BACKSLASH', 'Z', 'X', 'C', 'V', 'B', 'N', 'M',
                    ',', '.', '/', 'RSHIFT', '*', 'LALT', 'SPACE', 'CAPS_LOCK',
                    'F1', 'F2', 'F3', 'F4', 'F5', 'F6', 'F7', 'F8', 'F9',
                    'F10', 'NUM_LOCK', 'SCROLL_LOCK', 'NUMPAD_7', 'NUMPAD_8',
                    'NUMPAD_9', 'NUMPAD_-', 'NUMPAD_4', 'NUMPAD_5', 'NUMPAD_6',
                    'NUMPAD_+', 'NUMPAD_1', 'NUMPAD_2', 'NUMPAD_3', 'NUMPAD_0',
                    'NUMPAD_.', 'F11', 'F12', 'PRINT_SCREEN', 'PAUSE',
                    'NUMPAD_ENTER', 'RCTRL', 'NUMPAD_/', 'RALT', 'HOME', 'UP',
                    'PAGE_UP', 'LEFT', 'RIGHT', 'END', 'DOWN', 'PAGE_DOWN',
                    'INSERT', 'DELETE', 'LWINDOWS', 'RWINDOWS', 'MENU'}.
        """
        key_press(key)

    def key_release(self, key):
        """Simulates a key release event.

        Sends a scancode to the computer to report which key has been released.
        Some games use DirectInput devices, and respond only to scancodes, not
        virtual key codes. You can simulate DirectInput key releases using this
        method. A call to the key_release() method usually follows a call to
        the key_press() method of the same key.

        :param key: A string indicating which key to be released.
        """
        key_release(key)

    def touch(self, pos, **kwargs):
        """
        Perform mouse click action

        References:
            https://pywinauto.readthedocs.io/en/latest/code/pywinauto.mouse.html

        Args:
            pos: coordinates where to click
            **kwargs: optional arguments

        Examples:
            >>> from airtest.core.api import connect_device
            >>> dev = connect_device("Windows:///")
            >>> dev.touch((100, 100))  # absolute coordinates
            >>> dev.touch((0.5, 0.5))  # relative coordinates

        Returns:
            None

        """
        duration = kwargs.get("duration", 0.01)
        right_click = kwargs.get("right_click", False)
        button = "right" if right_click else "left"
        steps = kwargs.get("steps", 1)
        offset = kwargs.get("offset", 0)

        start = self._action_pos(win32api.GetCursorPos())
        ori_end = get_absolute_coordinate(pos, self)
        end = self._action_pos(ori_end)
        start_x, start_y = self._fix_op_pos(start)
        end_x, end_y = self._fix_op_pos(end)

        interval = float(duration) / steps
        time.sleep(interval)

        for i in range(1, steps):
            x = int(start_x + (end_x-start_x) * i / steps)
            y = int(start_y + (end_y-start_y) * i / steps)
            self.mouse.move(coords=(x, y))
            time.sleep(interval)

        self.mouse.move(coords=(end_x, end_y))

        for i in range(1, offset+1):
            self.mouse.move(coords=(end_x+i, end_y+i))
            time.sleep(0.01)

        for i in range(offset):
            self.mouse.move(coords=(end_x+offset-i, end_y+offset-i))
            time.sleep(0.01)

        self.mouse.press(button=button, coords=(end_x, end_y))
        time.sleep(duration)
        self.mouse.release(button=button, coords=(end_x, end_y))
        return ori_end

    def double_click(self, pos):
        ori_pos = get_absolute_coordinate(pos, self)
        coords = self._fix_op_pos(self._action_pos(ori_pos))
        self.mouse.double_click(coords=coords)
        return ori_pos

    def swipe(self, p1, p2, duration=0.8, steps=5, button="left"):
        """
        Perform swipe (mouse press and mouse release)

        Args:
            p1: start point
            p2: end point
            duration: time interval to perform the swipe action
            steps: size of the swipe step
            button: mouse button to press, 'left', 'right' or 'middle', default is 'left'

        Examples:
            >>> from airtest.core.api import connect_device
            >>> dev = connect_device("Windows:///")
            >>> dev.swipe((100, 100), (200, 200), duration=0.5)
            >>> dev.swipe((0.1, 0.1), (0.2, 0.2), duration=0.5)

        Returns:
            None

        """
        # 设置坐标时相对于整个屏幕的坐标:
        ori_from = get_absolute_coordinate(p1, self)
        ori_to = get_absolute_coordinate(p2, self)
        from_x, from_y = self._fix_op_pos(self._action_pos(ori_from))
        to_x, to_y = self._fix_op_pos(self._action_pos(ori_to))

        interval = float(duration) / (steps + 1)
        self.mouse.press(coords=(from_x, from_y), button=button)
        time.sleep(interval)
        for i in range(1, steps):
            self.mouse.move(coords=(
                int(from_x + (to_x - from_x) * i / steps),
                int(from_y + (to_y - from_y) * i / steps),
            ))
            time.sleep(interval)
        for i in range(10):
            self.mouse.move(coords=(to_x, to_y))
        time.sleep(interval)
        self.mouse.release(coords=(to_x, to_y), button=button)
        return ori_from, ori_to

    def mouse_move(self, pos):
        """Simulates a `mousemove` event.

        Known bug:
            Due to a bug in the pywinauto module, users might experience \
            off-by-one errors when it comes to the exact coordinates of \
            the position on screen.

        :param pos: A tuple (x, y), where x and y are x and y coordinates of
                    the screen to move the mouse to, respectively.
        """
        if not isinstance(pos, tuple) or len(pos) != 2:  # pos is not a 2-tuple
            raise ValueError('invalid literal for mouse_move: {}'.format(pos))
        try:
            self.mouse.move(coords=self._action_pos(pos))
        except ValueError:  # in case where x, y are not numbers
            raise ValueError('invalid literal for mouse_move: {}'.format(pos))

    def mouse_down(self, button='left'):
        """Simulates a `mousedown` event.

        :param button: A string indicating which mouse button to be pressed.
                       Available mouse button options are:
                       {'left', 'middle', 'right'}.
        """
        buttons = {'left', 'middle', 'right'}
        if not isinstance(button, str) or button not in buttons:
            raise ValueError('invalid literal for mouse_down(): {}'.format(button))
        else:
            coords = self._action_pos(win32api.GetCursorPos())
            self.mouse.press(button=button, coords=coords)

    def mouse_up(self, button='left'):
        """Simulates a `mouseup` event.

        A call to the mouse_up() method usually follows a call to the
        mouse_down() method of the same mouse button.

        :param button: A string indicating which mouse button to be released.
        """
        buttons = {'left', 'middle', 'right'}
        if not isinstance(button, str) or button not in buttons:
            raise ValueError('invalid literal for mouse_up(): {}'.format(button))
        else:
            coords = self._action_pos(win32api.GetCursorPos())
            self.mouse.release(button=button, coords=coords)

    def start_app(self, path, *args, **kwargs):
        """
        Start the application

        Args:
            path: full path to the application
            kwargs: reference: https://pywinauto.readthedocs.io/en/latest/code/pywinauto.application.html#pywinauto.application.Application.start

        Returns:
            None

        """
        self.app = self._app.start(path, **kwargs)

    def stop_app(self, pid):
        """
        Stop the application

        Args:
            pid: process ID of the application to be stopped

        Returns:
            None

        """
        self._app.connect(process=pid).kill()

    @require_app
    def set_foreground(self):
        """
        Bring the window foreground

        Returns:
            None

        """
        self._top_window.set_focus()

    set_focus = set_foreground

    def get_rect(self):
        """
        Get rectangle

        Returns:
            win32structures.RECT

        """
        if self.app and self._top_window:
            return self._top_window.rectangle()
        else:
            return RECT(right=win32api.GetSystemMetrics(0), bottom=win32api.GetSystemMetrics(1))

    @require_app
    def get_title(self):
        """
        Get the window title

        Returns:
            window title

        """
        return self._top_window.texts()

    @require_app
    def get_pos(self):
        """
        Get the window position coordinates

        Returns:
            coordinates of topleft corner of the window (left, top)

        """
        rect = self.get_rect()
        return (rect.left, rect.top)

    @require_app
    def move(self, pos):
        """
        Move window to given coordinates

        Args:
            pos: coordinates (x, y) where to move the window

        Returns:
            None

        """
        self._top_window.MoveWindow(x=pos[0], y=pos[1])

    @require_app
    def kill(self):
        """
        Kill the application

        Returns:
            None

        """
        self.app.kill()

    def set_clipboard(self, text):
        """
        Set clipboard content

        Args:
            text: text to be set to clipboard

        Examples:

            >>> from airtest.core.api import connect_device
            >>> dev = connect_device("Windows:///")
            >>> dev.set_clipboard("hello world")
            >>> print(dev.get_clipboard())
            'hello world'
            >>> dev.paste()  # paste the clipboard content

        Returns:
            None

        """
        try:
            win32clipboard.OpenClipboard()
        except pywintypes.error as e:
            # pywintypes.error: (5, 'OpenClipboard', '拒绝访问。')
            # sometimes clipboard is locked by other process, retry after 0.1s
            time.sleep(0.1)
            win32clipboard.OpenClipboard()
        try:
            win32clipboard.EmptyClipboard()
            win32clipboard.SetClipboardData(win32clipboard.CF_UNICODETEXT, text)
        finally:
            win32clipboard.CloseClipboard()

    def get_clipboard(self):
        """
        Get clipboard content

        Returns:
            clipboard content

        """
        try:
            win32clipboard.OpenClipboard()
            return win32clipboard.GetClipboardData()
        finally:
            win32clipboard.CloseClipboard()

    def paste(self):
        """
        Perform paste action

        Returns:
            None

        """
        self.keyevent("^v")

    def _action_pos(self, pos):
        if self.app:
            pos = self._windowpos_to_screenpos(pos)
        pos = (int(pos[0]), int(pos[1]))
        return pos

    # @property
    # def handle(self):
    #     return self._top_window.handle

    @property
    def focus_rect(self):
        return self._focus_rect

    @focus_rect.setter
    def focus_rect(self, value):
        # set focus rect to get rid of window border
        assert len(value) == 4, "focus rect must be in [left, top, right, bottom]"
        self._focus_rect = value

    def get_current_resolution(self):
        rect = self.get_rect()
        w = (rect.right + self._focus_rect[2]) - (rect.left + self._focus_rect[0])
        h = (rect.bottom + self._focus_rect[3]) - (rect.top + self._focus_rect[1])
        return w, h

    def _windowpos_to_screenpos(self, pos):
        """
        Convert given position relative to window topleft corner to screen coordinates

        Args:
            pos: coordinates (x, y)

        Returns:
            converted position coordinates

        """
        rect = self.get_rect()
        pos = (int((pos[0] + rect.left + self._focus_rect[0]) * self._dpifactor),
               int((pos[1] + rect.top + self._focus_rect[1]) * self._dpifactor))
        return pos

    def get_ip_address(self):
        """
        Return default external ip address of the windows os.

        Returns:
             :py:obj:`str`: ip address
        """
        ifaces = psutil.net_if_addrs()
        # 常见的虚拟网卡名称关键词
        virtual_iface_keywords = ['vEthernet', 'VirtualBox', 'VMware', 'Hyper-V', 'Wireless', 'VPN', 'Loopback']

        for interface_name, iface_addresses in ifaces.items():
            if any(keyword in interface_name for keyword in virtual_iface_keywords):
                continue

            for iface_address in iface_addresses:
                # 检查IPV4地址
                if iface_address.family == socket.AF_INET:
                    # 检查是否为自动获取的APIPA地址
                    if not iface_address.address.startswith('169.254'):
                        # 返回第一个非APIPA的IPV4地址
                        return iface_address.address

        return None


    def start_recording(self, max_time=1800, output=None, fps=10,
                        snapshot_sleep=0.001, orientation=0, max_size=None, *args, **kwargs):
        """
        Start recording the device display

        Args:
            max_time: maximum screen recording time, default is 1800
            output: ouput file path
            mode: the backend write video, choose in ["ffmpeg"]
                ffmpeg: ffmpeg-python backend, higher compression rate.
            fps: frames per second will record
            snapshot_sleep: sleep time for each snapshot.
            orientation: 1: portrait, 2: landscape, 0: rotation.
            max_size: max size of the video frame, e.g.800, default is None. Smaller sizes lead to lower system load.

        Returns:
            save_path: path of video file

        Examples:

            Record 30 seconds of video and export to the current directory test.mp4:

            >>> from airtest.core.api import connect_device, sleep
            >>> dev = connect_device("Windows:///")
            >>> save_path = dev.start_recording(output="test.mp4")
            >>> sleep(30)
            >>> dev.stop_recording()
            >>> print(save_path)

            You can specify max_size to limit the video's maximum width/length. Smaller video sizes result in lower CPU load.

            >>> dev.start_recording(output="test.mp4", max_size=800)

        Note:
            1 Don't resize the app window duraing recording, the recording region will be limited by first frame.
            2 If recording still working after app crash, it will continuing write last frame before the crash. 

        """
        if fps > 10 or fps < 1:
            LOGGING.warning("fps should be between 1 and 10, becuase of the recording effiency")
            if fps > 10:
                fps = 10
            if fps < 1:
                fps = 1

        if hasattr(self, 'recorder'):
            if self.recorder.is_running():
                LOGGING.warning("recording is already running, please don't call again")
                return None
        
        logdir = "./"
        if not ST.LOG_DIR is None:
            logdir = ST.LOG_DIR
        if output is None:
            save_path = os.path.join(logdir, "screen_%s.mp4"%(time.strftime("%Y%m%d%H%M%S", time.localtime())))
        else:
            if os.path.isabs(output):
                save_path = output
            else:
                save_path = os.path.join(logdir, output)

        max_size = get_max_size(max_size)
        def get_frame():
            frame = self.snapshot()
            
            if max_size is not None:
                frame = resize_by_max(frame, max_size)
            return frame

        self.recorder = ScreenRecorder(
            save_path, get_frame, fps=fps,
            snapshot_sleep=snapshot_sleep, orientation=orientation)
        self.recorder.stop_time = max_time
        self.recorder.start()
        LOGGING.info("start recording screen to {}, don't close or resize the app window".format(save_path))
        return save_path

    def stop_recording(self,):
        """
        Stop recording the device display. Recoding file will be kept in the device.

        """
        LOGGING.info("stopping recording")
        self.recorder.stop()
        return None



# coding=utf-8
import traceback
import uuid

from airtestProject.poco.utils import six


class NoSuchMethod(Exception):
    def __init__(self, name, available_methods):
        msg = 'No such method "{}". Available methods {}'.format(name, available_methods)
        super(NoSuchMethod, self).__init__(msg)


class StdRpcReactor(object):
    def __init__(self):
        super(StdRpcReactor, self).__init__()
        self.slots = {}  # method name -> method
        self.pending_response = {}  # rid -> result

    def register(self, name, method):
        if not callable(method):
            raise ValueError('Argument `method` should be a callable object. Got {}'.format(repr(method)))
        if name in self.slots:
            raise ValueError('"{}" already registered. {}'.format(name, repr(self.slots[name])))

        self.slots[name] = method

    def dispatch(self, name, *args, **kwargs):
        method = self.slots.get(name)
        if not method:
            raise NoSuchMethod(name, self.slots.keys())

        return method(*args, **kwargs)

    def handle_request(self, req):
        ret = {
            'id': req['id'],
            'jsonrpc': req['jsonrpc'],
        }

        method = req['method']
        params = req['params']
        try:
            result = self.dispatch(method, *params)
            ret['result'] = result
        except Exception as e:
            ret['error'] = {
                'message': '{}\n\n|--- REMOTE TRACEBACK ---|\n{}|--- REMOTE TRACEBACK END ---|'
                           .format(six.text_type(e), traceback.format_exc())
            }

        return ret

    def handle_response(self, res):
        id = res['id']
        self.pending_response[id] = res

    def build_request(self, method, *args, **kwargs):
        rid = six.text_type(uuid.uuid4())
        ret = {
            'id': rid,
            'jsonrpc': '2.0',
            'method': method,
            'params': args or kwargs or [],
        }
        self.pending_response[rid] = None
        return ret

    def get_result(self, rid):
        return self.pending_response.get(rid)



import os
import re
from xml.etree.ElementTree import fromstring
from io import BytesIO
from zipfile import ZipFile
from lxml import etree

from openpyxl.packaging.relationship import get_rels_path, get_dependents
from openpyxl.xml.constants import SHEET_DRAWING_NS, REL_NS, IMAGE_NS
from openpyxl.drawing.image import *

"""
author liuhuatong
des 获取图片路径，把excel表内嵌的图片另存起来，返回图片路径信息
date 2024/4/29
"""

def parse_element(element):
    """
    获取指定节点数据
    """
    data = {}
    xdr_namespace = "{%s}" % SHEET_DRAWING_NS
    targets = level_order_traversal(element, xdr_namespace + "nvPicPr")

    for target in targets:
        # 是一个cellimage
        cNvPr = embed = ""
        for child in target:
            if child.tag == xdr_namespace + "nvPicPr":
                cNvPr = child[0].attrib["name"]
            elif child.tag == xdr_namespace + "blipFill":
                _rel_embed = "{%s}embed" % REL_NS
                embed = child[0].attrib[_rel_embed]

        if cNvPr:
            data[cNvPr] = embed

    return data


def level_order_traversal(root, flag):
    """
    层次遍历，查找目标节点
    """
    queue = [root]
    targets = []
    while queue:
        node = queue.pop(0)
        children = [child.tag for child in node]
        if flag in children:
            targets.append(node)
            continue

        for child in node:
            queue.append(child)

    return targets


def handle_images(deps, archive, project, dir_name) -> []:
    """
    将图片二进制内容另存为
    """
    images = {}
    # if not PILImage:  # Pillow not installed, drop img
    #     return img

    for dep in deps:
        if dep.Type != IMAGE_NS:
            msg = "{0} 不支持的图像格式".format(dep.Type)
            print(msg)
            continue

        try:
            image_io = archive.read(dep.target)
            image = Image(BytesIO(image_io))
        except OSError:
            msg = "图像 {0} 将被删除，因为它无法读取".format(dep.target)
            print(msg)
            continue
        if image.format.upper() == "WMF":  # cannot save
            msg = "不支持 ｛0｝ 图像格式，因此正在删除该图像d".format(image.format)
            print(msg)
            continue
        image.embed = dep.id  # 文件rId
        image.target = dep.target  # 文件地址

        # 找到图片对象后，获取图片数据
        image_data = image_io

        image_name = image.target.split("/")[-1]
        # 使用文件名和'wb'模式写入图片数据
        images_path = "..//images//{}//{}".format(project, dir_name)
        if not os.path.exists(images_path):  # 判断是否存在文件夹如果不存在则创建为文件夹
            os.makedirs(images_path)

        with open("{}//{}".format(images_path, image_name), 'wb') as img_file:
            img_file.write(image_data)

        images.setdefault(image.embed, os.path.abspath("{}//{}".format(images_path, image_name)))

    return images


def parseElement(element):
    """
    获取指定节点数据
    """
    et = etree.fromstring(element)
    workSheets = et.findall("{*}sheetData")
    table = {}
    tablePlus = {}
    workSheetNames = []
    for workSheet in workSheets:
        # @*[local-name() = 'Name']获取当前节点下属性为Name的值
        rowName = workSheet.findall("{*}row")  # 返回的是个列表，用索引取出值
        workSheetNames.append(rowName)
        c = workSheet.xpath(".//*[local-name() = 'c']")
        for cell in c:
            v = cell.findall("{*}f")
            if len(v) != 0:
                match = re.search(r'\((.*?)\)', cell.findall("{*}f")[0].text)
                if match:
                    content_inside_brackets = match.group(1)
                    ID = content_inside_brackets.split(",")[0].strip('"')
                    table.setdefault(cell.attrib['r'], ID)
                    tablePlus.setdefault(cell.findall("{*}f")[0].text, ID)

    return table, tablePlus


def get_id_name(project="zg", PARSE_FILE_PATH = 'ZgTestCases1.xlsx', page=1):
    """获取excel表图片，id与文件路径的对应信息
    :param project: 项目
    :param PARSE_FILE_PATH: excel路径
    :param page: excel工坐表下标索引
    :return:
    """
    CELLIMAGE_PATH = "xl/cellimages.xml"
    sheetName = "xl/worksheets/sheet{}.xml".format(page+1)
    archive = ZipFile(PARSE_FILE_PATH, "r")

    sheet = archive.read(sheetName)
    src = archive.read(CELLIMAGE_PATH)  # 打开cellImage.xml文件
    deps = get_dependents(archive, get_rels_path(CELLIMAGE_PATH))  # 解析cellImage.xml._rel文件
    image_rels = handle_images(deps=deps.Relationship, archive=archive, project=project, dir_name=PARSE_FILE_PATH.split("\\")[-1].split(".")[0])

    node = fromstring(src)
    tabel, tabelPlus = parseElement(sheet)
    cellimages_xml = parse_element(node)

    for cnvpr, embed in cellimages_xml.items():
        cellimages_xml[cnvpr] = image_rels.get(embed)

    for ln, embed in tabel.items():
        tabel[ln] = cellimages_xml.get(embed)

    for ln, embed in tabelPlus.items():
        tabelPlus[ln] = cellimages_xml.get(embed)
    archive.close()  # 关闭压缩文件对象，防止内存泄漏
    return tabelPlus

if __name__ == '__main__':

    # 这个数字传入是sheet的下标，第一个sheet，第二个
    get_id_name(project="zg", PARSE_FILE_PATH='ZgTestCases.xlsx', page=0)
